﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

namespace Xenon.Aims.SceneGraph
{


    public abstract class Actions_Coordmat
    {


        #region アクション
        //────────────────────────────────────────

        /// <summary>
        /// 離す横。
        /// </summary>
        public static void DetachesH(Point location, UcMain ucMain)
        {
            // ──────────
            // 該当シーンボックス
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                if (location.X - location.X % UcMain.CELL_SIZE <= scene.Bounds.X)
                {
                    // 移動確定
                    SceneBox sceneOld = scene.Clone(scene.Id);
                    scene.SourceBounds = new Rectangle(
                        scene.SourceBounds.X + UcMain.CELL_SIZE,
                        scene.SourceBounds.Y,
                        scene.SourceBounds.Width,
                        scene.SourceBounds.Height
                        );
                    UcMain.DiffContent(scene.Id, "シーンボックス#移動", scene.Diff(sceneOld));
                }
            }

            // ──────────
            // 該当接続線
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                Cable cableOld = cable.Clone(cable.Id);

                // 始端、終端
                for (int i = 0; i < 2; i++)
                {
                    if (location.X - location.X % UcMain.CELL_SIZE <= cable.Bounds[i].X)
                    {
                        // 移動確定
                        cable.SourceBounds[i] = new Rectangle(
                            cable.SourceBounds[i].X + UcMain.CELL_SIZE,
                            cable.SourceBounds[i].Y,
                            cable.SourceBounds[i].Width,
                            cable.SourceBounds[i].Height
                            );
                    }
                }

                // 中間点1～2
                for (int i = 0; i < 2; i++)
                {
                    if (location.X - location.X % UcMain.CELL_SIZE <= cable.BoundsNode[i].X)
                    {
                        // 移動確定
                        cable.SourceBoundsNode[i] = new Point(
                            cable.SourceBoundsNode[i].X + UcMain.CELL_SIZE,
                            cable.SourceBoundsNode[i].Y
                            );
                    }
                }

                UcMain.DiffContent(cable.Id, "接続線#移動", cable.Diff(cableOld));
            }

            // ──────────
            // 該当画像
            // ──────────
            foreach (ImageBox ib in ucMain.Boxes.IndexImageboxList)
            {
                if (location.X - location.X % UcMain.CELL_SIZE <= ib.Bounds.X)
                {
                    // 移動確定
                    ImageBox ibOld = ib.Clone(ib.Id);
                    ib.SourceLocation = new Point(
                        ib.SourceLocation.X + UcMain.CELL_SIZE,
                        ib.SourceLocation.Y
                        );
                    UcMain.DiffContent(ib.Id, "画像#移動", ib.Diff(ibOld));
                }
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// 離す縦。
        /// </summary>
        public static void DetachesV(Point location, UcMain ucMain)
        {
            // ──────────
            // 該当シーンボックス
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                if (location.Y - location.Y % UcMain.CELL_SIZE <= scene.Bounds.Y)
                {
                    // 移動確定
                    SceneBox sceneOld = scene.Clone(scene.Id);
                    scene.SourceBounds = new Rectangle(
                        scene.SourceBounds.X,
                        scene.SourceBounds.Y + UcMain.CELL_SIZE,
                        scene.SourceBounds.Width,
                        scene.SourceBounds.Height
                        );
                    UcMain.DiffContent(scene.Id, "シーンボックス#移動", scene.Diff(sceneOld));
                }
            }


            // ──────────
            // 該当接続線
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                Cable cableOld = cable.Clone(cable.Id);

                // 始端、終端
                for (int i = 0; i < 2; i++)
                {
                    if (location.Y - location.Y % UcMain.CELL_SIZE <= cable.Bounds[i].Y)
                    {
                        cable.SourceBounds[i] = new Rectangle(
                            cable.SourceBounds[i].X,
                            cable.SourceBounds[i].Y + UcMain.CELL_SIZE,
                            cable.SourceBounds[i].Width,
                            cable.SourceBounds[i].Height
                            );
                    }
                }

                // 中間点1～2
                for (int i = 0; i < 2; i++)
                {
                    if (location.Y - location.Y % UcMain.CELL_SIZE <= cable.BoundsNode[i].Y)
                    {
                        cable.SourceBoundsNode[i] = new Point(
                            cable.SourceBoundsNode[i].X,
                            cable.SourceBoundsNode[i].Y + UcMain.CELL_SIZE
                            );
                    }
                }

                UcMain.DiffContent(cable.Id, "接続線#移動", cable.Diff(cableOld));
            }


            // ──────────
            // 該当画像
            // ──────────
            foreach (ImageBox ib in ucMain.Boxes.IndexImageboxList)
            {
                if (location.Y - location.Y % UcMain.CELL_SIZE <= ib.Bounds.Y)
                {
                    // 移動確定
                    ImageBox ibOld = ib.Clone(ib.Id);
                    ib.SourceLocation = new Point(
                        ib.SourceLocation.X,
                        ib.SourceLocation.Y + UcMain.CELL_SIZE
                        );
                    UcMain.DiffContent(ib.Id, "シーンボックス#移動", ib.Diff(ibOld));
                }
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// 縮める横。
        /// </summary>
        public static void ShortenH(Point location, UcMain ucMain)
        {
            // ──────────
            // 該当シーンボックス
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                if (location.X - location.X % UcMain.CELL_SIZE <= scene.Bounds.X)
                {
                    SceneBox sceneOld = scene.Clone(scene.Id);
                    scene.SourceBounds = new Rectangle(
                        scene.SourceBounds.X - UcMain.CELL_SIZE,
                        scene.SourceBounds.Y,
                        scene.SourceBounds.Width,
                        scene.SourceBounds.Height
                        );
                    UcMain.DiffContent(scene.Id, "シーンボックス#移動", scene.Diff(sceneOld));
                }
            }


            // ──────────
            // 該当接続線
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                Cable cableOld = cable.Clone(cable.Id);

                // 始点、終点
                for (int i = 0; i < 2; i++)
                {
                    if (location.X - location.X % UcMain.CELL_SIZE <= cable.Bounds[i].X)
                    {
                        cable.SourceBounds[i] = new Rectangle(
                            cable.SourceBounds[i].X - UcMain.CELL_SIZE,
                            cable.SourceBounds[i].Y,
                            cable.SourceBounds[i].Width,
                            cable.SourceBounds[i].Height
                            );
                    }
                }

                // 中間点1～2
                for (int i = 0; i < 2; i++)
                {
                    if (location.X - location.X % UcMain.CELL_SIZE <= cable.BoundsNode[i].X)
                    {
                        cable.SourceBoundsNode[i] = new Point(
                            cable.SourceBoundsNode[i].X - UcMain.CELL_SIZE,
                            cable.SourceBoundsNode[i].Y
                            );
                    }
                }

                UcMain.DiffContent(cable.Id, "接続線#移動", cable.Diff(cableOld));
            }


            // ──────────
            // 該当画像ボックス
            // ──────────
            foreach (ImageBox scene in ucMain.Boxes.IndexImageboxList)
            {
                if (location.X - location.X % UcMain.CELL_SIZE <= scene.Bounds.X)
                {
                    ImageBox old = scene.Clone(scene.Id);
                    scene.SourceLocation = new Point(
                        scene.SourceLocation.X - UcMain.CELL_SIZE,
                        scene.SourceLocation.Y
                        );
                    UcMain.DiffContent(scene.Id, "シーンボックス#移動", scene.Diff(old));
                }
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// 縮める縦。
        /// </summary>
        public static void ShortenV(Point location, UcMain ucMain)
        {
            // ──────────
            // 該当シーンボックス
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                if (location.Y - location.Y % UcMain.CELL_SIZE <= scene.Bounds.Y)
                {
                    SceneBox sceneOld = scene.Clone(scene.Id);
                    scene.SourceBounds = new Rectangle(
                        scene.SourceBounds.X,
                        scene.SourceBounds.Y - UcMain.CELL_SIZE,
                        scene.SourceBounds.Width,
                        scene.SourceBounds.Height
                        );
                    UcMain.DiffContent(scene.Id, "シーンボックス#移動", scene.Diff(sceneOld));
                }
            }


            // ──────────
            // 該当接続線
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                Cable cableOld = cable.Clone(cable.Id);

                // 始点、終点
                for (int i = 0; i < 2; i++)
                {
                    if (location.Y - location.Y % UcMain.CELL_SIZE <= cable.Bounds[i].Y)
                    {
                        cable.SourceBounds[i] = new Rectangle(
                            cable.SourceBounds[i].X,
                            cable.SourceBounds[i].Y - UcMain.CELL_SIZE,
                            cable.SourceBounds[i].Width,
                            cable.SourceBounds[i].Height
                            );
                    }
                }

                // 中間点1～2
                for (int i = 0; i < 2; i++)
                {
                    if (location.Y - location.Y % UcMain.CELL_SIZE <= cable.BoundsNode[i].Y)
                    {
                        cable.SourceBoundsNode[i] = new Point(
                            cable.SourceBoundsNode[i].X,
                            cable.SourceBoundsNode[i].Y - UcMain.CELL_SIZE
                            );
                    }
                }

                UcMain.DiffContent(cable.Id, "接続線#移動", cable.Diff(cableOld));
            }


            // ──────────
            // 該当画像ボックス
            // ──────────
            foreach (ImageBox ib in ucMain.Boxes.IndexImageboxList)
            {
                if (location.Y - location.Y % UcMain.CELL_SIZE <= ib.Bounds.Y)
                {
                    ImageBox old = ib.Clone(ib.Id);
                    ib.SourceLocation = new Point(
                        ib.SourceLocation.X,
                        ib.SourceLocation.Y - UcMain.CELL_SIZE
                        );
                    UcMain.DiffContent(ib.Id, "画像ボックス#移動", ib.Diff(old));
                }
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// マット拡張左。
        /// </summary>
        public static void ExtendL(UcMain ucMain)
        {
            // ──────────
            // 全シーン
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                scene.SourceBounds = new Rectangle(
                    scene.SourceBounds.X + ucMain.CoordMat.SourceBounds.Width,
                    scene.SourceBounds.Y,
                    scene.SourceBounds.Width,
                    scene.SourceBounds.Height
                    );
            }


            // ──────────
            // 全接続線
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                for (int i = 0; i < 2; i++)
                {
                    cable.SourceBounds[i] = new Rectangle(
                        cable.SourceBounds[i].X + ucMain.CoordMat.SourceBounds.Width,
                        cable.SourceBounds[i].Y,
                        cable.SourceBounds[i].Width,
                        cable.SourceBounds[i].Height
                        );
                }
            }


            // ──────────
            // 全画像
            // ──────────
            foreach (ImageBox ib in ucMain.Boxes.IndexImageboxList)
            {
                ib.SourceLocation = new Point(
                    ib.SourceLocation.X + ucMain.CoordMat.SourceBounds.Width,
                    ib.SourceLocation.Y
                    );
            }
        }

        //────────────────────────────────────────


        /// <summary>
        /// マット縮小左。
        /// </summary>
        public static void ReductL(UcMain ucMain)
        {
            // ──────────
            // 全シーン
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                scene.SourceBounds = new Rectangle(
                    scene.SourceBounds.X - ucMain.CoordMat.SourceBounds.Width,
                    scene.SourceBounds.Y,
                    scene.SourceBounds.Width,
                    scene.SourceBounds.Height
                    );
            }


            // ──────────
            // 全接続線
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                for (int i = 0; i < 2; i++)
                {
                    cable.SourceBounds[i] = new Rectangle(
                        cable.SourceBounds[i].X - ucMain.CoordMat.SourceBounds.Width,
                        cable.SourceBounds[i].Y,
                        cable.SourceBounds[i].Width,
                        cable.SourceBounds[i].Height
                        );
                }
            }


            // ──────────
            // 全画像
            // ──────────
            foreach (ImageBox ib in ucMain.Boxes.IndexImageboxList)
            {
                ib.SourceLocation = new Point(
                    ib.SourceLocation.X - ucMain.CoordMat.SourceBounds.Width,
                    ib.SourceLocation.Y
                    );
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// マット縮小上。
        /// </summary>
        public static void ReductT(UcMain ucMain)
        {
            // ──────────
            // 全シーン
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                scene.SourceBounds = new Rectangle(
                    scene.SourceBounds.X,
                    scene.SourceBounds.Y - ucMain.CoordMat.SourceBounds.Height,
                    scene.SourceBounds.Width,
                    scene.SourceBounds.Height
                    );
            }


            // ──────────
            // 全接続線
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                for (int i = 0; i < 2; i++)
                {
                    cable.SourceBounds[i] = new Rectangle(
                        cable.SourceBounds[i].X,
                        cable.SourceBounds[i].Y - ucMain.CoordMat.SourceBounds.Height,
                        cable.SourceBounds[i].Width,
                        cable.SourceBounds[i].Height
                        );
                }
            }


            // ──────────
            // 全画像
            // ──────────
            foreach (ImageBox ib in ucMain.Boxes.IndexImageboxList)
            {
                ib.SourceLocation = new Point(
                    ib.SourceLocation.X,
                    ib.SourceLocation.Y - ucMain.CoordMat.SourceBounds.Height
                    );
            }
        }

        //────────────────────────────────────────
        //────────────────────────────────────────
        #endregion


    }


}
