﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;

namespace Xenon.Aims.SceneGraph
{

    /// <summary>
    /// シーンボックス、
    /// 接続線、
    /// 画像
    /// </summary>
    public class Boxes
    {



        #region 生成と破棄
        //────────────────────────────────────────

        public void Clear()
        {
            this.IndexSceneboxList.Clear();
            this.IndexCableList.Clear();
            this.IndexImageboxList.Clear();
        }

        public Boxes()
        {
            this.IndexSceneboxList = new List<SceneBox>();
            this.IndexCableList = new List<Cable>();
            this.IndexImageboxList = new List<ImageBox>();
        }

        //────────────────────────────────────────
        #endregion



        #region アクション
        //────────────────────────────────────────

        /// <summary>
        /// グリッドのずれ修正。
        /// </summary>
        public void JustGrid(UcMain uiMain)
        {
            // ──────────
            // 全シーン
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                if (
                    0 != (scene.SourceBounds.X - uiMain.CoordMat.SourceBounds.X) % UcMain.CELL_SIZE ||
                    0 != (scene.SourceBounds.Y - uiMain.CoordMat.SourceBounds.Y) % UcMain.CELL_SIZE
                    )
                {
                    scene.SourceBounds = new Rectangle(
                        scene.SourceBounds.X - (scene.SourceBounds.X - uiMain.CoordMat.SourceBounds.X) % UcMain.CELL_SIZE,
                        scene.SourceBounds.Y - (scene.SourceBounds.Y - uiMain.CoordMat.SourceBounds.Y) % UcMain.CELL_SIZE,
                        scene.SourceBounds.Width,
                        scene.SourceBounds.Height
                        );
                }
            }

            // ──────────
            // 全接続線
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                for (int i = 0; i < 2; i++)
                {
                    if (
                        0 != (cable.SourceBounds[i].X - uiMain.CoordMat.SourceBounds.X) % UcMain.CELL_SIZE ||
                        0 != (cable.SourceBounds[i].Y - uiMain.CoordMat.SourceBounds.Y) % UcMain.CELL_SIZE
                        )
                    {
                        cable.SourceBounds[i] = new Rectangle(
                            cable.SourceBounds[i].X - (cable.SourceBounds[i].X - uiMain.CoordMat.SourceBounds.X) % UcMain.CELL_SIZE,
                            cable.SourceBounds[i].Y - (cable.SourceBounds[i].Y - uiMain.CoordMat.SourceBounds.Y) % UcMain.CELL_SIZE,
                            cable.SourceBounds[i].Width,
                            cable.SourceBounds[i].Height
                            );
                    }
                }
            }

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="g"></param>
        /// <param name="ucMain"></param>
        /// <param name="r"></param>
        /// <param name="c"></param>
        public void Screenshot(Graphics g, UcMain ucMain, int r, int c)
        {

            //レイヤーで分かれているので、オブジェクトを１周ループするだけではありません。

            // ──────────
            // 全接続線BC　描画
            // ──────────
            //スクリーンショットでは、円を描きません。
            //foreach (Cable cable in this.CableList)
            //{
            //    cable.ScreenshotBackcircle(g, ucMain, r, c);
            //}

            // ──────────
            // 全画像　描画
            // ──────────
            foreach (ImageBox ib in this.IndexImageboxList)
            {
                ib.Screenshot(g, ucMain, r, c);
            }

            // ──────────
            // 全接続線L　描画
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                cable.ScreenshotLine(g, ucMain, r, c);
            }


            // ──────────
            // 全シーン　描画
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                scene.Screenshot(g, ucMain, r, c);
            }



            // ──────────
            // 全接続線F　描画
            // ──────────
            //スクリーンショットでは、円を描きません。
            //foreach (Cable cable in this.CableList)
            //{
            //    cable.ScreenshotMousemark(g,ucMain,r,c);
            //}

        }

        /// <summary>
        /// マット拡張上。
        /// </summary>
        public void ExtendT(UcMain uiMain)
        {
            // ──────────
            // 全シーン
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                scene.SourceBounds = new Rectangle(
                    scene.SourceBounds.X,
                    scene.SourceBounds.Y + uiMain.CoordMat.SourceBounds.Height,
                    scene.SourceBounds.Width,
                    scene.SourceBounds.Height
                    );
            }

            // ──────────
            // 全接続線
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                for (int i = 0; i < 2; i++)
                {
                    cable.SourceBounds[i] = new Rectangle(
                        cable.SourceBounds[i].X,
                        cable.SourceBounds[i].Y + uiMain.CoordMat.SourceBounds.Height,
                        cable.SourceBounds[i].Width,
                        cable.SourceBounds[i].Height
                        );
                }
            }

        }


        public void Paint(Graphics g, UcMain ucMain)
        {
            // ──────────
            // 画像
            // ──────────
            {
                foreach (ImageBox ib in this.IndexImageboxList)
                {
                    ib.Paint(g, ucMain);
                }
            }

            // ──────────
            // 全接続線BC
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                cable.PaintBackCircle(g);
            }

            // ──────────
            // 全接続線L
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                cable.PaintLine(g);
            }


            // ──────────
            // シーンボックス
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                scene.Paint(g, ucMain);
            }

            // ──────────
            // 全接続線F
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                cable.PaintMouseMark(g,ucMain);
            }

        }

        public void ClearSelection()
        {
            // ──────────
            // 全シーン　選択解除
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                scene.IsSelected = false;
            }

            // ──────────
            // 全接続線　選択解除
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                cable.IsSelected[0] = false;
                cable.IsSelected[1] = false;
                cable.IsSelectedNode[0] = false;
                cable.IsSelectedNode[1] = false;
            }

            // ──────────
            // 全画像　選択解除
            // ──────────
            foreach (ImageBox ib in this.IndexImageboxList)
            {
                ib.IsSelected = false;
            }
        }


        public void CheckMouseOver(Point location, ref bool forcedOff)
        {
            // ──────────
            // 優先順1　全接続線
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                cable.CheckMouseOver(0, location, ref forcedOff);
                cable.CheckMouseOver(1, location, ref forcedOff);
                cable.CheckMouseOverNode(0, location, ref forcedOff);
                cable.CheckMouseOverNode(1, location, ref forcedOff);
            }

            // ──────────
            // 優先順2　全シーン
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                scene.CheckMouseOver(location, ref forcedOff);
            }

        }

        public void MoveSelectedScene(UcMain ucMain)
        {
            // ──────────
            // シーン
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                if (scene.IsSelected)
                {
                    // 移動確定
                    SceneBox sceneOld = scene.Clone(scene.Id);
                    scene.SourceBounds = new Rectangle(
                        scene.SourceBounds.X + ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE,
                        scene.SourceBounds.Y + ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE,
                        scene.SourceBounds.Width,
                        scene.SourceBounds.Height
                        );
                    UcMain.DiffContent(scene.Id, "選択シーンボックス#移動確定", scene.Diff(sceneOld));

                    // 移動量クリアー
                    scene.Movement = new Rectangle();
                }
            }
        }

        public void MoveSelectedCable(UcMain ucMain)
        {
            // ──────────
            // 接続線
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                Cable cableOld = cable.Clone(cable.Id);

                for (int i = 0; i < 2; i++)
                {
                    if (cable.IsSelected[i])
                    {
                        // 移動確定
                        cable.SourceBounds[i] = new Rectangle(
                            cable.SourceBounds[i].X + ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE,
                            cable.SourceBounds[i].Y + ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE,
                            cable.SourceBounds[i].Width,
                            cable.SourceBounds[i].Height
                            );

                        // 移動量クリアー
                        cable.Movement[i] = new Rectangle();
                    }
                }

                for (int i = 0; i < 2; i++)
                {
                    if (cable.IsSelectedNode[i])
                    {
                        cable.SourceBoundsNode[i] = new Point(
                            cable.SourceBoundsNode[i].X + ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE,
                            cable.SourceBoundsNode[i].Y + ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE
                            );

                        // 移動量クリアー
                        cable.MovementNode[i] = new Point();
                    }
                }

                UcMain.DiffContent(cable.Id, "選択接続線#移動確定", cable.Diff(cableOld));
            }
        }

        public void MoveSelectedImage(UcMain ucMain)
        {
            // ──────────
            // 画像
            // ──────────
            foreach (ImageBox ib in this.IndexImageboxList)
            {
                if (ib.IsSelected)
                {
                    // 移動確定
                    ImageBox sceneOld = ib.Clone(ib.Id);
                    ib.SourceLocation = new Point(
                        ib.SourceLocation.X + ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE,
                        ib.SourceLocation.Y + ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE
                        );
                    UcMain.DiffContent(ib.Id, "選択シーンボックス#移動確定", ib.Diff(sceneOld));

                    // 移動量クリアー
                    ib.Movement = new Point();
                }
            }
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="ucMain"></param>
        /// <param name="isMovedCoordmat">マットを動かしたのなら真。</param>
        public static void SetMovementToAllSceneboxes(UcMain ucMain, bool isMovedCoordmat)
        {
            // ──────────
            // 全シーン　移動量セット
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                int x;
                int y;
                if (isMovedCoordmat)
                {
                    // マットを動かしているので、マットの移動量＝要素の移動量です。
                    x = ucMain.Movement.X;
                    y = ucMain.Movement.Y;
                }
                else
                {
                    // ずれ補正
                    //
                    //　　移動量は、グリッドサイズの倍数となる。
                    //
                    x = ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE;
                    y = ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE;
                }

                scene.Movement = new Rectangle(x, y, 0, 0);
            }
        }

        public static void SetMovementToSelectedScenebox(UcMain ucMain)
        {
            // ──────────
            // 選択シーン　移動量セット
            // ──────────
            foreach (SceneBox scene in ucMain.Boxes.IndexSceneboxList)
            {
                if (scene.IsSelected)
                {
                    // ずれ補正
                    //
                    //　　移動量は、グリッドサイズの倍数となる。
                    //
                    scene.Movement = new Rectangle(
                        ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE,
                        ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE,
                        0, 0);
                }
            }
        }

        public static void SetMovementToSelectedCable(UcMain ucMain)
        {
            // ──────────
            // 選択接続線　移動量セット
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                for (int i = 0; i < 2; i++)
                {
                    if (cable.IsSelected[i])
                    {
                        cable.Movement[i] = new Rectangle(
                            ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE,
                            ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE,
                            0, 0);
                    }
                }

                for (int i = 0; i < 2; i++)
                {
                    if (cable.IsSelectedNode[i])
                    {
                        cable.MovementNode[i] = new Point(
                            ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE,
                            ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE
                            );
                    }
                }
            }
        }

        public static void SetMovementToAllCables(UcMain ucMain, bool isMovedCoordmat)
        {
            // ──────────
            // 全接続線　移動量セット
            // ──────────
            foreach (Cable cable in ucMain.Boxes.IndexCableList)
            {
                // ──────────
                // [0]起点　[1]終点
                // ──────────
                for (int i = 0; i < 2; i++)
                {
                    int x;
                    int y;
                    if (isMovedCoordmat)
                    {
                        // マットを動かしているので、マットの移動量＝要素の移動量です。
                        x = ucMain.Movement.X;
                        y = ucMain.Movement.Y;
                    }
                    else
                    {
                        x = ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE;
                        y = ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE;
                    }

                    cable.Movement[i] = new Rectangle(
                        x,
                        y,
                        0, 0);
                }

                for (int i = 0; i < 2; i++)
                {
                    int x;
                    int y;
                    if (isMovedCoordmat)
                    {
                        // マットを動かしているので、マットの移動量＝要素の移動量です。
                        x = ucMain.Movement.X;
                        y = ucMain.Movement.Y;
                    }
                    else
                    {
                        x = ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE;
                        y = ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE;
                    }

                    cable.MovementNode[i] = new Point(
                        x,
                        y);
                }
            }
        }

        public static void SetMovementToAllImageboxes(UcMain ucMain, bool isMovedCoordmat)
        {
            // ──────────
            // 全画像　移動量セット
            // ──────────
            foreach (ImageBox ib in ucMain.Boxes.IndexImageboxList)
            {
                int x;
                int y;
                if (isMovedCoordmat)
                {
                    // マットを動かしているので、マットの移動量＝要素の移動量です。
                    x = ucMain.Movement.X;
                    y = ucMain.Movement.Y;
                }
                else
                {
                    // ずれ補正
                    //
                    //　　移動量は、グリッドサイズの倍数となる。
                    //
                    x = ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE;
                    y = ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE;
                }

                ib.Movement = new Point(x, y);
            }
        }

        public static void SetMovementToSelectedImagebox(UcMain ucMain)
        {
            // ──────────
            // 選択画像　移動量セット
            // ──────────
            foreach (ImageBox ib in ucMain.Boxes.IndexImageboxList)
            {
                if (ib.IsSelected)
                {
                    // ずれ補正
                    //
                    //　　移動量は、グリッドサイズの倍数となる。
                    //
                    ib.Movement = new Point(
                        ucMain.Movement.X - ucMain.Movement.X % UcMain.CELL_SIZE,
                        ucMain.Movement.Y - ucMain.Movement.Y % UcMain.CELL_SIZE);
                }
            }
        }


        /// <summary>
        /// シーンの移動。
        /// </summary>
        /// <param name="uiMain"></param>
        /// <param name="isMoveCoordMat"></param>
        public void MoveAllScenes(UcMain uiMain, bool isMoveCoordMat)
        {
            // ──────────
            // 全シーン　移動確定
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                int x;
                int y;
                if (isMoveCoordMat)
                {
                    x = scene.SourceBounds.X + uiMain.Movement.X;
                    y = scene.SourceBounds.Y + uiMain.Movement.Y;
                }
                else
                {
                    x = scene.SourceBounds.X + uiMain.Movement.X - uiMain.Movement.X % UcMain.CELL_SIZE;
                    y = scene.SourceBounds.Y + uiMain.Movement.Y - uiMain.Movement.Y % UcMain.CELL_SIZE;
                }

                // 移動確定
                SceneBox sceneOld = scene.Clone(scene.Id);
                scene.SourceBounds = new Rectangle(
                    x,
                    y,
                    scene.SourceBounds.Width,
                    scene.SourceBounds.Height
                    );
                UcMain.DiffContent(scene.Id, "シーンボックス#移動確定", scene.Diff(sceneOld));

                // 移動量クリアー
                scene.Movement = new Rectangle();
            }
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="uiMain"></param>
        /// <param name="isMoveCoordMat"></param>
        public void MoveAllImages(UcMain uiMain, bool isMoveCoordMat)
        {
            // ──────────
            // 全画像　移動確定
            // ──────────
            foreach (ImageBox ib in this.IndexImageboxList)
            {
                int x;
                int y;
                if (isMoveCoordMat)
                {
                    x = ib.SourceLocation.X + uiMain.Movement.X;
                    y = ib.SourceLocation.Y + uiMain.Movement.Y;
                }
                else
                {
                    x = ib.SourceLocation.X + uiMain.Movement.X - uiMain.Movement.X % UcMain.CELL_SIZE;
                    y = ib.SourceLocation.Y + uiMain.Movement.Y - uiMain.Movement.Y % UcMain.CELL_SIZE;
                }

                // 移動確定
                ImageBox ibOld = ib.Clone(ib.Id);
                ib.SourceLocation = new Point(
                    x,
                    y
                    );
                UcMain.DiffContent(ib.Id, "画像ボックス#移動確定", ib.Diff(ibOld));

                // 移動量クリアー
                ib.Movement = new Point();
            }
        }

        /// <summary>
        /// 接続線の移動
        /// </summary>
        public void MoveCable(UcMain uiMain, bool isMoveCoordMat)
        {
            // 全ての接続線を、移動量分だけ移動します。
            foreach (Cable cable in this.IndexCableList)
            {
                Cable cableOld = cable.Clone(cable.Id);

                // ──────────
                // [0]起点　[1]終点
                // ──────────
                for (int i = 0; i < 2; i++)
                {
                    int x;
                    int y;
                    if (isMoveCoordMat)
                    {
                        x = cable.SourceBounds[i].X + uiMain.Movement.X;
                        y = cable.SourceBounds[i].Y + uiMain.Movement.Y;
                    }
                    else
                    {
                        x = cable.SourceBounds[i].X + uiMain.Movement.X - uiMain.Movement.X % UcMain.CELL_SIZE;
                        y = cable.SourceBounds[i].Y + uiMain.Movement.Y - uiMain.Movement.Y % UcMain.CELL_SIZE;
                    }

                    // 移動確定
                    cable.SourceBounds[i] = new Rectangle(
                        x,
                        y,
                        cable.SourceBounds[i].Width,
                        cable.SourceBounds[i].Height
                        );

                    // 移動量クリアー
                    cable.Movement[i] = new Rectangle();
                }

                // ──────────
                // 中間点[0～1]
                // ──────────
                for (int i = 0; i < 2; i++)
                {
                    int x;
                    int y;
                    if (isMoveCoordMat)
                    {
                        x = cable.SourceBoundsNode[i].X + uiMain.Movement.X;
                        y = cable.SourceBoundsNode[i].Y + uiMain.Movement.Y;
                    }
                    else
                    {
                        x = cable.SourceBoundsNode[i].X + uiMain.Movement.X - uiMain.Movement.X % UcMain.CELL_SIZE;
                        y = cable.SourceBoundsNode[i].Y + uiMain.Movement.Y - uiMain.Movement.Y % UcMain.CELL_SIZE;
                    }

                    // 移動確定
                    cable.SourceBoundsNode[i] = new Point(
                        x,
                        y
                        );

                    // 移動量クリアー
                    cable.MovementNode[i] = new Point();
                }

                UcMain.DiffContent(cable.Id, "接続線#移動確定", cable.Diff(cableOld));
            }
        }

        //────────────────────────────────────────
        #endregion



        #region アクション_ファイル
        //────────────────────────────────────────

        public void Save(StringBuilder sb)
        {
            // ──────────
            // 全シーン
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                scene.Save(sb);
            }

            // ──────────
            // 全接続線
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                cable.Save(sb);
            }

            // ──────────
            // 全画像
            // ──────────
            foreach (ImageBox ib in this.IndexImageboxList)
            {
                ib.Save(sb);
            }
        }

        //────────────────────────────────────────
        #endregion



        #region 判定
        //────────────────────────────────────────

        public bool IsHit(Point location)
        {
            bool isHit = false;

            // ──────────
            // 全シーン
            // ──────────
            foreach (SceneBox scene in this.IndexSceneboxList)
            {
                if (scene.IsHit(location))
                {
                    isHit = true;
                    break;
                }
            }

            // ──────────
            // 全接続線
            // ──────────
            foreach (Cable cable in this.IndexCableList)
            {
                if (cable.IsHit(0, location))
                {
                    isHit = true;
                    break;
                }

                if (cable.IsHit(1, location))
                {
                    isHit = true;
                    break;
                }

                if (cable.IsHitNode(0, location))
                {
                    isHit = true;
                    break;
                }

                if (cable.IsHitNode(1, location))
                {
                    isHit = true;
                    break;
                }
            }

            return isHit;
        }

        //────────────────────────────────────────
        #endregion



        #region プロパティー
        //────────────────────────────────────────

        private List<SceneBox> indexSceneboxList_;
        /// <summary>
        /// シーンボックスだけを抜き出した一覧。
        /// </summary>
        public List<SceneBox> IndexSceneboxList
        {
            get
            {
                return this.indexSceneboxList_;
            }
            set
            {
                this.indexSceneboxList_ = value;
            }
        }

        //────────────────────────────────────────

        private List<Cable> indexCableList_;
        /// <summary>
        /// 接続線だけを抜き出した一覧
        /// </summary>
        public List<Cable> IndexCableList
        {
            get
            {
                return this.indexCableList_;
            }
            set
            {
                this.indexCableList_ = value;
            }
        }

        //────────────────────────────────────────

        private List<ImageBox> indexImageboxList_;
        /// <summary>
        /// ドラッグ＆ドロップした画像だけを抜き出した一覧。
        /// </summary>
        public List<ImageBox> IndexImageboxList
        {
            get
            {
                return indexImageboxList_;
            }
            set
            {
                indexImageboxList_ = value;
            }
        }

        //────────────────────────────────────────
        #endregion


    }
}
