﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;
using System.Windows.Forms;

namespace Xenon.Aims.SceneGraph
{

    /// <summary>
    /// ドラッグ＆ドロップ
    /// </summary>
    public class DragAndDrop
    {



        #region 用意
        //────────────────────────────────────────

        public delegate void DropAction(object sender, DragEventArgs e, string fileName);

        //────────────────────────────────────────
        #endregion



        #region 生成と破棄
        //────────────────────────────────────────

        public DragAndDrop()
        {
            this.DraggingCursorMove = Cursor.Current;
            this.DraggingCursorCopy = Cursor.Current;
            this.DraggingCursorLink = Cursor.Current;
            this.DraggingCursorNone = Cursor.Current;

            this.OnImageDropped = (object sender, DragEventArgs e, string fileName) => { };
        }

        //────────────────────────────────────────
        #endregion



        #region アクション
        //────────────────────────────────────────

        public void Load()
        {

        }

        //────────────────────────────────────────
        #endregion



        #region イベントハンドラー
        //────────────────────────────────────────

        /// <summary>
        /// ドラッグ＆ドロップ中に発生します。
        /// ドラッグをキャンセルするのに使います。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnQueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            // ドラッグ中にマウスの右ボタンが押されていれば、ドラッグをキャンセルします。

            // "2"は、マウスの右ボタンを表します。何故か定数はないようです。
            if ((e.KeyState & 2) == 2)
            {
                e.Action = DragAction.Cancel;
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// コントロールの上で、マウス・ボタンが押下されたとき。
        /// 
        /// マウス・カーソルの座標位置を記憶します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnMouseDown(object sender, MouseEventArgs e)
        {
            // マウスの左ボタンが押下されている時、ドラッグを実行中にします。

            // マウスの左ボタンが押されている時。
            if (e.Button == MouseButtons.Left)
            {
                // 選択時。
                this.SetMouseDownLocation(e.Location);// new Point(e.X, e.Y)

                // ここでは　ドラッグ＆ドロップしません。
            }
            // マウスの左ボタン以外が押されたとき。
            else
            {
                this.SetMouseDownLocation( Point.Empty);
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// コントロールの上で、マウス・ボタンが放されたとき。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnMouseUp(object sender, MouseEventArgs e)
        {
            this.SetMouseDownLocation( Point.Empty);
        }

        //────────────────────────────────────────

        /// <summary>
        /// コントロールの上で、マウス・カーソルが移動したとき。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnMouseMove(object sender, MouseEventArgs e)
        {
            //━━━━━
            //コントロールからアイテムを持ち出す時
            //━━━━━
        }

        //────────────────────────────────────────

        /// <summary>
        /// データが、コントロールの上にドロップされたとき。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnDragDrop(object sender, DragEventArgs e)
        {
            //デリゲーター this.panel_Clear_OnImagesDropInit(sender, e);

            //ドロップされたデータの形式を調べます。

            // ファイルドロップ
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                // ファイルであれば。
                string[] fileNames = (string[])e.Data.GetData(DataFormats.FileDrop, false);

                StringBuilder debugMessage = new StringBuilder();
                debugMessage.Append("ドロップされたファイル数=[");
                debugMessage.Append(fileNames.Length);
                debugMessage.Append("]\r\n");

                foreach (string fileName in fileNames)
                {
                    try
                    {
                        //━━━━━
                        // 画像ファイルであった場合
                        //━━━━━
                        this.OnImageDropped(sender, e, fileName);//デリゲーター
                    }
                    catch (Exception)
                    {
                        // 画像ではなかった場合。

                        // 無視します。
                    }

                    debugMessage.Append(fileName);
                    debugMessage.Append("\r\n");
                }

                //デリゲーター this.panel_Report_OnImagesDropFinished(sender, e, debugMessage.ToString(), "ファイル・パスのようなものがパネルの上に落とされました。");
            }
            else
            {
                //デリゲーター this.panel_Report_OnImagesDropFinished(sender, e, "", "何かパネルの上に落とされましたが、文字列としても画像としても読み取れませんでした。");
            }
            // 文字列または画像のどちらにも読み取れないデータは無視します。

        }

        //────────────────────────────────────────

        /// <summary>
        /// データが、パネルの上にエンターされたとき。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnDragEnter(object sender, DragEventArgs e)
        {
            System.Console.WriteLine("★★★★★★★★OnDragEnter");


            //デリゲーター this.panel_Clear_OnDragEnterInit(sender, e);

            // ドラッグされてきたデータの形式を一覧します。
            {
                StringBuilder text = new StringBuilder();
                text.Append("ドラッグされてきたデータの形式の個数=[");
                text.Append(e.Data.GetFormats().Length);
                text.Append("]\r\n");
                foreach (string format in e.Data.GetFormats())
                {
                    text.Append(format);
                    text.Append("\r\n");
                }
                System.Console.WriteLine(text.ToString());
            }


            string debugStatusEntersText;
            string debugStatusReceivesTxt;
            // ドラッグされてきたデータの形式を調べます。

            // ファイルドロップ
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                debugStatusEntersText = "ファイルのようなものがパネルの上にドラッグされてきました。";

                object droppedObj = e.Data.GetData(DataFormats.FileDrop, false);

                if (droppedObj is string[])
                {
                    // ファイル名であれば。
                    string[] fileNames = (string[])droppedObj;

                    // 画像ファイルが含まれているかどうか。
                    bool imageContained = false;

                    foreach (string fileName in fileNames)
                    {
                        try
                        {
                            // 画像ファイルとして読み込めるか、
                            // そうでなければ例外を投げます。
                            // ヌルになることはないかと思います。
                            Bitmap droppedBitmap = new Bitmap(fileName);

                            // 画像ファイルが含まれていた場合。
                            imageContained = true;
                        }
                        catch (Exception)
                        {
                            // 画像ファイルでなければ無視して続行。
                        }
                    }

                    if (imageContained)
                    {
                        // ドロップした時の効果を Copy として見えるようにします。
                        e.Effect = DragDropEffects.Copy;
                        debugStatusReceivesTxt = "コピーの受け入れ態勢をとります。画像として読み取れるファイルがありました。";
                    }
                    else
                    {
                        // ドロップできないように見えるようにします。
                        e.Effect = DragDropEffects.None;
                        debugStatusReceivesTxt = "受け入れない態勢をとります。画像として読み取れるファイルがありませんでした。";
                    }
                }
                else
                {
                    debugStatusReceivesTxt = "";
                }
            }
            else
            {
                debugStatusEntersText = "何かパネルの上にドラッグされてきました。";

                // ファイル・ドロップでなければ、ドロップできないように見えるようにします。
                e.Effect = DragDropEffects.None;
                debugStatusReceivesTxt = "受け入れない態勢をとります。ファイルとして読み取れません。";
            }

            //デリゲーター this.panel_Report_OnDragEnterFinished(sender, e, debugFormatsText, debugStatusEntersText, debugStatusReceivesTxt);
        }

        //────────────────────────────────────────
        #endregion




        #region プロパティー
        //────────────────────────────────────────

        private Cursor draggingCursorMove;
        /// <summary>
        /// ドラッグ時のカーソルの画像。移動時。
        /// </summary>
        public Cursor DraggingCursorMove
        {
            get
            {
                return draggingCursorMove;
            }
            set
            {
                draggingCursorMove = value;
            }
        }

        //────────────────────────────────────────

        private Cursor draggingCursorCopy;
        /// <summary>
        /// ドラッグ時のカーソルの画像。コピー時。
        /// </summary>
        public Cursor DraggingCursorCopy
        {
            get
            {
                return draggingCursorCopy;
            }
            set
            {
                draggingCursorCopy = value;
            }
        }

        //────────────────────────────────────────

        private Cursor draggingCursorLink;
        /// <summary>
        /// ドラッグ時のカーソルの画像。ショートカット作成時。
        /// </summary>
        public Cursor DraggingCursorLink
        {
            get
            {
                return draggingCursorLink;
            }
            set
            {
                draggingCursorLink = value;
            }
        }

        //────────────────────────────────────────

        private Cursor draggingCursorNone;
        /// <summary>
        /// ドラッグ時のカーソルの画像。禁止時。
        /// </summary>
        public Cursor DraggingCursorNone
        {
            get
            {
                return draggingCursorNone;
            }
            set
            {
                draggingCursorNone = value;
            }
        }

        //────────────────────────────────────────

        private Point mouseDownLocation_;
        /// <summary>
        /// マウス・ボタンが押された時の座標位置
        /// </summary>
        public Point MouseDownLocation
        {
            get
            {
                return mouseDownLocation_;
            }
        }

        public void SetMouseDownLocation(Point value)
        {
            this.mouseDownLocation_ = value;
        }

        //────────────────────────────────────────

        private DropAction onImageDropped_;
        /// <summary>
        /// 画像ファイル＆ドロップ時・実行プログラム。
        /// </summary>
        public DropAction OnImageDropped
        {
            get
            {
                return this.onImageDropped_;
            }
            set
            {
                this.onImageDropped_ = value;
            }
        }

        //────────────────────────────────────────
        #endregion


    }
}
