﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Drawing;
using System.Xml;

namespace Xenon.Aims.SceneGraph
{


    public class ImageBox
    {


        #region 生成と破棄
        //────────────────────────────────────────

        public ImageBox(int id)
        {
            this.Id = id;
            this.Clear();
        }

        public void Clear()
        {
            this.Src = "";
            this.IsVisible = true;
            this.SourceLocation = new Point();
            this.Movement = new Point();
        }

        public ImageBox Clone(int idForClone)
        {
            ImageBox ib = new ImageBox(idForClone);

            // Id はコピーしません。

            ib.SetImage( this.Image);

            ib.IsSelected = this.IsSelected;
            ib.IsVisible = this.IsVisible;

            ib.Movement = new Point(
                this.Movement.X,
                this.Movement.Y
                );

            ib.SourceLocation = new Point(
                this.SourceLocation.X,
                this.SourceLocation.Y
                );

            ib.Src = this.Src;

            return ib;
        }

        //────────────────────────────────────────
        #endregion



        #region アクション
        //────────────────────────────────────────

        public void Screenshot(Graphics g, UcMain ucMain, int r, int c)
        {
            Point old = new Point(
                this.SourceLocation.X,
                this.SourceLocation.Y
                );

            this.SourceLocation = new Point(
                this.SourceLocation.X - c * ucMain.CoordMat.SourceBounds.Width - ucMain.CoordMat.SourceBounds.X,
                this.SourceLocation.Y - r * ucMain.CoordMat.SourceBounds.Height - ucMain.CoordMat.SourceBounds.Y
                );

            this.Paint(g, ucMain);

            this.SourceLocation = old;
        }

        //────────────────────────────────────────

        /// <summary>
        /// どこが変更されたのかを情報表示します。
        /// </summary>
        /// <param name="old"></param>
        /// <returns></returns>
        public string Diff(ImageBox old)
        {
            StringBuilder sb = new StringBuilder();

            if (this.Id != old.Id)
            {
                sb.Append(" Id(" + old.Id + "→" + this.Id + ")");
            }

            if (this.Image != old.Image)
            {
                sb.Append(" Image(" + old.Image + "→" + this.Image + ")");
            }

            if (
                this.Movement.X != old.Movement.X ||
                this.Movement.Y != old.Movement.Y
                )
            {
                sb.Append(" Movement(" + old.Movement.X + "," + old.Movement.Y + ")" +
                    "→(" + this.Movement.X + "," + this.Movement.Y + ")");
            }

            if (
                this.SourceLocation.X != old.SourceLocation.X ||
                this.SourceLocation.Y != old.SourceLocation.Y
                )
            {
                sb.Append(" SourceLocation(" + old.SourceLocation.X + "," + old.SourceLocation.Y + ")" +
                    "→(" + this.SourceLocation.X + "," + this.SourceLocation.Y + ")");
            }

            return sb.ToString();
        }


        public void Paint(Graphics g, UcMain ucMain)
        {

            //────────────────────────────────────────
            // 移動前の残像
            //────────────────────────────────────────


            Rectangle bounds2;
            if (this.Movement.X != 0 || this.Movement.Y != 0)
            {
                // 少しでも移動しているなら。

                bounds2 = new Rectangle(
                    (int)(this.SourceLocation.X),
                    (int)(this.SourceLocation.Y),
                    (int)(this.Image.Width),
                    (int)(this.Image.Height)
                    );

                // 画像を半透明にします。
                Image img;
                System.Drawing.Imaging.ImageAttributes ia;
                {
                    img = (Image)this.Image.Clone();


                    //ColorMatrixオブジェクトの作成
                    System.Drawing.Imaging.ColorMatrix cm =
                        new System.Drawing.Imaging.ColorMatrix();
                    //ColorMatrixの行列の値を変更して、アルファ値が0.5に変更されるようにする
                    cm.Matrix00 = 1;
                    cm.Matrix11 = 1;
                    cm.Matrix22 = 1;
                    cm.Matrix33 = 0.5F;
                    cm.Matrix44 = 1;

                    //ImageAttributesオブジェクトの作成
                    ia = new System.Drawing.Imaging.ImageAttributes();
                    //ColorMatrixを設定する
                    ia.SetColorMatrix(cm);
                }

                //ImageAttributesを使用して画像を描画
                g.DrawImage(
                    img,
                    bounds2,
                    0, 0, img.Width, img.Height,
                    GraphicsUnit.Pixel, ia);

                //リソースを解放する
                img.Dispose();
            }

            //────────────────────────────────────────
            // 移動後
            //────────────────────────────────────────

            bounds2 = new Rectangle(
                (int)(this.SourceLocation.X + this.Movement.X),
                (int)(this.SourceLocation.Y + this.Movement.Y),
                (int)(this.Image.Width),
                (int)(this.Image.Height)
            );

            g.DrawImage(this.Image, bounds2);

            // 選択表示
            if (this.IsSelected)
            {
                g.DrawRectangle(Pens.YellowGreen, bounds2);
            }
            else
            {
            }

            //g.DrawImage(
            //    this.Image,
            //    new Rectangle(
            //        this.SourceLocation.X,
            //        this.SourceLocation.Y,
            //        this.Image.Width,
            //        this.Image.Height
            //        ));

            // デバッグ設定
            if (UcMain.VisibleId)
            {
                g.DrawString(
                    this.Id.ToString(),
                    new Font(ucMain.Font.Name, 12.0f),
                    Brushes.Red,
                    this.Bounds.Location
                    );
            }

        }

        //────────────────────────────────────────
        #endregion






        #region アクション_ファイル
        //────────────────────────────────────────

        public void Save(StringBuilder sb)
        {
            sb.Append("  <img");
            sb.Append(" src=\""+this.Src+"\"");
            sb.Append(" x=\"" + this.Bounds.X + "\" y=\"" + this.Bounds.Y + "\" width=\"" + this.Bounds.Width + "\" height=\"" + this.Bounds.Height + "\"");
            sb.Append(" selected=\"" + this.IsSelected + "\"");
            sb.Append(" visible=\"" + this.IsVisible + "\"");
            sb.Append(" />");
            sb.Append(Environment.NewLine);

            //ystem.Console.WriteLine("シーン：　座標（" + this.Bounds.X + "," + this.Bounds.Y + "）　タイトル（" + this.Title + "）");
        }

        //────────────────────────────────────────

        public void Load(XmlElement xe)
        {
            this.Clear();

            int x;
            int y;
            int w;
            int h;
            bool isSelected;
            bool isVisible;

            //━━━━━
            //文字列型
            //━━━━━
            {
                string s;
                s = xe.GetAttribute("x");
                int.TryParse(s, out x);
                s = xe.GetAttribute("y");
                int.TryParse(s, out y);
                s = xe.GetAttribute("width");
                int.TryParse(s, out w);
                s = xe.GetAttribute("height");
                int.TryParse(s, out h);
            }

            //━━━━━
            //論理型
            //━━━━━
            {
                string s;
                s = xe.GetAttribute("selected");
                bool.TryParse(s, out isSelected);
                s = xe.GetAttribute("visible");
                bool.TryParse(s, out isVisible);
            }


            this.Src = xe.GetAttribute("src");//ファイル名
            this.SetImage(new Bitmap(this.Src));
            this.SourceLocation = new Point(x, y);


            this.IsSelected = isSelected;
            this.IsVisible = isVisible;
        }

        //────────────────────────────────────────
        #endregion



        #region 判定
        //────────────────────────────────────────

        /// <summary>
        /// 表示しているとき、指定座標が境界内なら真。
        /// </summary>
        /// <param name="location"></param>
        /// <returns></returns>
        public bool IsHit(Point location)
        {
            return this.IsVisible && this.Bounds.Contains(location);
        }

        //────────────────────────────────────────
        #endregion



        #region プロパティー
        //────────────────────────────────────────

        private int id;
        /// <summary>
        /// コンテンツに割り当てられる通し番号です。物を特定するために重複させない数字です。
        /// アプリケーション起動時～終了までの間だけ変わりません。
        /// </summary>
        public int Id
        {
            get
            {
                return id;
            }
            set
            {
                id = value;
            }
        }

        //────────────────────────────────────────

        private string src_;

        public string Src
        {
            get
            {
                return src_;
            }
            set
            {
                src_ = value;
            }
        }

        //────────────────────────────────────────

        private Image image_;
        /// <summary>
        /// 画像。
        /// </summary>
        public Image Image
        {
            get
            {
                return image_;
            }
        }

        public void SetImage(Image value)
        {
            this.image_ = value;
        }

        //────────────────────────────────────────

        private Point sourceLocation;
        /// <summary>
        /// 位置。
        /// </summary>
        public Point SourceLocation
        {
            get
            {
                return sourceLocation;
            }
            set
            {
                sourceLocation = value;
            }
        }

        //────────────────────────────────────────

        private Point movement;
        /// <summary>
        /// 動かした量。
        /// </summary>
        public Point Movement
        {
            get
            {
                return movement;
            }
            set
            {
                movement = value;
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// 移動後の境界線。
        /// </summary>
        public Rectangle Bounds
        {
            get
            {
                Rectangle bounds;

                if (this.Image != null)
                {
                    bounds = new Rectangle(
                        this.SourceLocation.X + this.Movement.X,
                        this.SourceLocation.Y + this.Movement.Y,
                        this.Image.Width,
                        this.Image.Height
                        );
                }
                else
                {
                    bounds = new Rectangle(
                        this.SourceLocation.X + this.Movement.X,
                        this.SourceLocation.Y + this.Movement.Y,
                        1,
                        1
                        );
                }

                return bounds;
            }
        }

        //────────────────────────────────────────

        private bool isSelected;
        /// <summary>
        /// 選択中。
        /// </summary>
        public bool IsSelected
        {
            get
            {
                return isSelected;
            }
            set
            {
                isSelected = value;
            }
        }

        //────────────────────────────────────────

        private bool isVisible;
        /// <summary>
        /// 表示の有無。
        /// </summary>
        public bool IsVisible
        {
            get
            {
                return this.isVisible;
            }
            set
            {
                this.isVisible = value;
            }
        }

        //────────────────────────────────────────
        #endregion


    }


}
