﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;

using System.IO;

namespace Xenon.Aims.SceneGraph
{
    public partial class UcMain : UserControl
    {



        #region 用意
        //────────────────────────────────────────

        public static readonly int CELL_SIZE = 20;

        //────────────────────────────────────────
        #endregion



        #region 生成と破棄
        //────────────────────────────────────────


        public void Clear()
        {
            this.Boxes.Clear();

            this.coordMatRepeatX = 1;
            this.coordMatRepeatY = 1;
            this.verticalCursorX = 0;
            this.isVerticalCursorEnable = false;
            this.horizontalCursorY = 0;
            this.isHorizontalCursorEnable = false;

            this.Font = new Font("ＭＳ ゴシック", 12.0f, FontStyle.Regular);
        }

        public UcMain()
        {
            this.Boxes = new Boxes();

            this.Clear();

            //━━━━━
            // デバッグ設定
            //━━━━━
            UcMain.VisibleId = false;// true;

            this.dragAndDrop = new DragAndDrop();
            this.dragAndDrop.OnImageDropped = (object sender, DragEventArgs e, string fileName) =>
            {
                System.Console.WriteLine("★画像ファイルをドロップしました。fileName[" + fileName + "]");

                //タイムスタンプ
                string timestamp;
                {
                    StringBuilder s = new StringBuilder();
                    DateTime now = System.DateTime.Now;
                    s.Append(String.Format("{0:D4}", now.Year));
                    s.Append(String.Format("{0:D2}", now.Month));
                    s.Append(String.Format("{0:D2}", now.Day));
                    s.Append("_");
                    s.Append(String.Format("{0:D2}", now.Hour));
                    s.Append(String.Format("{0:D2}", now.Minute));
                    s.Append("_");
                    s.Append(String.Format("{0:D2}", now.Second));
                    s.Append(String.Format("{0:D3}", now.Millisecond));
                    this.fileNameCounter++;
                    timestamp = s.ToString();
                }
                string newFileName = "saveImg/" + timestamp + "_" + this.fileNameCounter + ".png";

                if (!Directory.Exists(Directory.GetParent(newFileName).Name))
                {
                    // saveImg フォルダーがなければ、作ります。
                    Directory.CreateDirectory(Directory.GetParent(newFileName).Name);
                }


                System.Console.WriteLine("★saveImgフォルダーにファイルをコピーします。newFileName[" + newFileName + "]");
                File.Copy(fileName, newFileName);

                System.Console.WriteLine("★画像を生成し、リストに入れます。");
                ImageBox ib = new ImageBox(this.NextId());
                ib.Src = newFileName;
                ib.SetImage(new Bitmap(fileName));
                ib.SourceLocation = this.PointToClient(new Point(
                    e.X - ib.Image.Width / 2,
                    e.Y - ib.Image.Height / 2
                    ));
                this.Boxes.IndexImageboxList.Add(ib);

                this.Refresh();
            };


            this.buttonAreaBottom = 3 * 50 + 5;
            this.clickedLocation = new Point();
            // 座標マット
            this.coordMat = new CoordMat(this.NextId());
            this.coordMat.IsSelected = true;
            this.coordMat.SourceBounds = new Rectangle(150, 150, this.coordMat.SourceBounds.Width, this.coordMat.SourceBounds.Height);


            this.buttons = new Dictionary<string, GraphicButton>();
            GraphicButton btn;

            //━━━━━
            //左上ボタン
            //━━━━━
            {
                //━━━━━
                // セーブボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "save";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Save.png";
                btn.Bounds = new Rectangle(0 * 50, 0, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    Actions_File.Save(this);
                };
                //━━━━━
                // ロードボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "load";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Load.png";
                btn.Bounds = new Rectangle(0 * 50, 1 * 50, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region ロード

                    // ──────────
                    // クリアー
                    // ──────────
                    this.Clear();
                    // シーンボックス
                    this.Boxes.Clear();


                    XmlDocument doc = new XmlDocument();
                    try
                    {
                        doc.Load("save.xml");
                        //ystem.Console.WriteLine("要素数="+doc.DocumentElement.ChildNodes.Count);

                        foreach (XmlNode xn in doc.DocumentElement.ChildNodes)
                        {
                            if (xn.NodeType == XmlNodeType.Element)
                            {
                                XmlElement xe = (XmlElement)xn;
                                string s;
                                bool b;
                                int x;
                                int y;
                                int w;
                                int h;
                                switch (xe.Name)
                                {
                                    // ──────────
                                    // UiMain
                                    // ──────────
                                    case "ui-main":
                                        s = xe.GetAttribute("coord-mat-repeat-x");
                                        int.TryParse(s, out x);
                                        this.coordMatRepeatX = x;
                                        s = xe.GetAttribute("coord-mat-repeat-y");
                                        int.TryParse(s, out y);
                                        this.coordMatRepeatY = y;

                                        //━━━━━
                                        //フォント
                                        //━━━━━
                                        {
                                            float fontSize;

                                            s = xe.GetAttribute("font-size");
                                            if (float.TryParse(s, out fontSize))
                                            {
                                                if (fontSize < 1)
                                                {
                                                    fontSize = 8.0f;// TODO:フォントサイズ
                                                }
                                            }
                                            else
                                            {
                                                fontSize = 8.0f;// TODO:フォントサイズ
                                            }
                                            //ystem.Console.WriteLine("fontSize=" + fontSize + "　this.FontSize=" + this.FontSize);

                                            this.Font = new Font(xe.GetAttribute("font-name"), fontSize, FontStyle.Regular);
                                        }

                                        break;

                                    // ──────────
                                    // 座標マット
                                    // ──────────
                                    case "coord-mat":
                                        this.coordMat.Load(xe);
                                        break;

                                    // ──────────
                                    // シーン
                                    // ──────────
                                    case "scene":
                                        SceneBox scene = new SceneBox(this.NextId());
                                        scene.Load(xe);
                                        this.Boxes.IndexSceneboxList.Add(scene);
                                        break;

                                    // ──────────
                                    // 接続線
                                    // ──────────
                                    case "cable":
                                        Cable cable = new Cable(this.NextId());
                                        cable.Load(xe);

                                        this.Boxes.IndexCableList.Add(cable);
                                        break;

                                    // ──────────
                                    // 画像
                                    // ──────────
                                    case "img":
                                        ImageBox ib = new ImageBox(this.NextId());
                                        ib.Load(xe);

                                        this.Boxes.IndexImageboxList.Add(ib);
                                        break;
                                }
                            }
                        }

                        //────────────────────────────────────────
                        // グリッドのずれ修正
                        //────────────────────────────────────────

                        // ──────────
                        // 全シーン
                        // ──────────
                        this.Boxes.JustGrid(this);
                    }
                    catch (Exception)
                    {
                        // セーブファイルが無いなど。
                    }

                    this.Refresh();

                    #endregion
                };
                //━━━━━
                // スクリーンショットボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "screenShot";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ScreenShot.png";
                btn.Bounds = new Rectangle(0 * 50 + 0, 2 * 50 + 0, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    Actions_File.ScreenShot(this);
                };
            }


            //━━━━━
            // 座標マットボタン
            //━━━━━
            {
                //━━━━━
                //大ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "coordMat";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_CoordMat.png";
                btn.Bounds = new Rectangle(0 * 75 + 100, 0, 75, 75);
                btn.IsSelected = true;
                btn.SwitchOnAction = () =>
                {
                    this.coordMat.IsSelected = true;
                    // ボタン表示
                    this.buttons["moveMat"].IsVisible = true;
                    this.buttons["extendT"].IsVisible = true;
                    this.buttons["extendR"].IsVisible = true;
                    this.buttons["extendB"].IsVisible = true;
                    this.buttons["extendL"].IsVisible = true;
                    this.buttons["reductT"].IsVisible = true;
                    this.buttons["reductR"].IsVisible = true;
                    this.buttons["reductB"].IsVisible = true;
                    this.buttons["reductL"].IsVisible = true;
                    this.buttons["detachesH"].IsVisible = true;
                    this.buttons["detachesV"].IsVisible = true;
                    this.buttons["shortenH"].IsVisible = true;
                    this.buttons["shortenV"].IsVisible = true;
                };
                btn.RadioOffAction = () =>
                {
                    this.coordMat.IsSelected = false;
                    // ボタン非表示
                    this.buttons["moveMat"].IsVisible = false;
                    this.buttons["extendT"].IsVisible = false;
                    this.buttons["extendR"].IsVisible = false;
                    this.buttons["extendB"].IsVisible = false;
                    this.buttons["extendL"].IsVisible = false;
                    this.buttons["reductT"].IsVisible = false;
                    this.buttons["reductR"].IsVisible = false;
                    this.buttons["reductB"].IsVisible = false;
                    this.buttons["reductL"].IsVisible = false;
                    this.buttons["detachesH"].IsVisible = false;
                    this.buttons["detachesV"].IsVisible = false;
                    this.buttons["shortenH"].IsVisible = false;
                    this.buttons["shortenV"].IsVisible = false;
                };


                //━━━━━
                // 移動ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "move";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Move.png";
                btn.Bounds = new Rectangle(2 * 50, 0 * 50 + 75, 50, 50);
                btn.IsSelected = true;
                //━━━━━
                // マット移動ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "moveMat";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_MoveMat.png";
                btn.Bounds = new Rectangle(3 * 50, 0 * 50 + 75, 50, 50);
                //━━━━━
                // 拡張上ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "extendT";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ExtendT.png";
                btn.Bounds = new Rectangle(4 * 50, 0 * 50 + 75, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region 拡張_上

                    if (this.coordMatRepeatY < int.MaxValue)
                    {
                        this.coordMatRepeatY++;

                        //────────────────────────────────────────
                        // 座標マット１個分下にずらします。
                        //────────────────────────────────────────

                        this.Boxes.ExtendT(this);

                        this.Refresh();
                    }

                    #endregion
                };
                //━━━━━
                // 拡張右ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "extendR";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ExtendR.png";
                btn.Bounds = new Rectangle(5 * 50, 0 * 50 + 75, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region 拡張_右

                    if (this.coordMatRepeatX < int.MaxValue)
                    {
                        this.coordMatRepeatX++;
                        this.Refresh();
                    }

                    #endregion
                };
                //━━━━━
                // 拡張下ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "extendB";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ExtendB.png";
                btn.Bounds = new Rectangle(6 * 50, 0 * 50 + 75, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region 拡張_下

                    if (this.coordMatRepeatY < int.MaxValue)
                    {
                        this.coordMatRepeatY++;
                        this.Refresh();
                    }

                    #endregion
                };
                //━━━━━
                // 拡張左ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "extendL";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ExtendL.png";
                btn.Bounds = new Rectangle(7 * 50, 0 * 50 + 75, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region 拡張_左

                    if (this.coordMatRepeatX < int.MaxValue)
                    {
                        this.coordMatRepeatX++;

                        //────────────────────────────────────────
                        // 座標マット１個分右にずらします。
                        //────────────────────────────────────────
                        Actions_Coordmat.ExtendL(this);
                        this.Refresh();
                    }

                    #endregion
                };
                //━━━━━
                // 縮小上ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "reductT";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ReductT.png";
                btn.Bounds = new Rectangle(8 * 50, 0 * 50 + 75, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region 縮小_上

                    if (1 < this.coordMatRepeatY)
                    {
                        this.coordMatRepeatY--;

                        //────────────────────────────────────────
                        // 座標マット１個分上にずらします。
                        //────────────────────────────────────────
                        Actions_Coordmat.ReductT(this);
                        this.Refresh();
                    }
                    #endregion
                };
                //━━━━━
                // 縮小右ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "reductR";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ReductR.png";
                btn.Bounds = new Rectangle(9 * 50, 0 * 50 + 75, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region 縮小_右
                    if (1 < this.coordMatRepeatX)
                    {
                        this.coordMatRepeatX--;
                        this.Refresh();
                    }
                    #endregion
                };
                //━━━━━
                // 縮小下ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "reductB";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ReductB.png";
                btn.Bounds = new Rectangle(10 * 50, 0 * 50 + 75, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region 縮小_下
                    if (1 < this.coordMatRepeatY)
                    {
                        this.coordMatRepeatY--;
                        this.Refresh();
                    }
                    #endregion
                };
                //━━━━━
                // 縮小左ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "reductL";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ReductL.png";
                btn.Bounds = new Rectangle(11 * 50, 0 * 50 + 75, 50, 50);
                btn.SwitchOnAction = () =>
                {
                    #region 縮小_左
                    if (1 < this.coordMatRepeatX)
                    {
                        this.coordMatRepeatX--;

                        //────────────────────────────────────────
                        // 座標マット１個分左にずらします。
                        //────────────────────────────────────────
                        Actions_Coordmat.ReductL(this);
                        this.Refresh();
                    }
                    #endregion
                };
                //━━━━━
                // 離す横
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "detachesH";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_DetachesH.png";
                btn.Bounds = new Rectangle(12 * 50, 0 * 50 + 75, 50, 50);
                //━━━━━
                // 離す縦
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "detachesV";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_DetachesV.png";
                btn.Bounds = new Rectangle(13 * 50, 0 * 50 + 75, 50, 50);
                //━━━━━
                // 縮める横
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "shortenH";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ShortenH.png";
                btn.Bounds = new Rectangle(14 * 50, 0 * 50 + 75, 50, 50);
                //━━━━━
                // 縮める縦
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "shortenV";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_ShortenV.png";
                btn.Bounds = new Rectangle(15 * 50, 0 * 50 + 75, 50, 50);
            }

            //━━━━━
            // シーンボタン
            //━━━━━
            {
                btn = new GraphicButton(this.NextId());
                btn.Name = "scene";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Scene.png";
                btn.Bounds = new Rectangle(1 * 75 + 100, 0, 75, 75);
                btn.SwitchOnAction = () =>
                {
                    // ボタン表示
                    this.buttons["text"].IsVisible = true;
                    this.buttons["create"].IsVisible = true;
                    this.buttons["copy"].IsVisible = true;
                    this.buttons["delete"].IsVisible = true;
                    this.buttons["connect"].IsVisible = true;
                    this.buttons["cable4"].IsVisible = true;
                    this.buttons["shape"].IsVisible = true;
                };
                btn.RadioOffAction = () =>
                {
                    // ボタン非表示
                    this.buttons["text"].IsVisible = false;
                    this.buttons["create"].IsVisible = false;
                    this.buttons["copy"].IsVisible = false;
                    this.buttons["delete"].IsVisible = false;
                    this.buttons["connect"].IsVisible = false;
                    this.buttons["cable4"].IsVisible = false;
                    this.buttons["shape"].IsVisible = false;
                };


                //━━━━━
                // テキストボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "text";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Text.png";
                btn.Bounds = new Rectangle(3 * 50, 0 * 50 + 75, 50, 50);
                btn.IsVisible = false;
                //━━━━━
                // 作成ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "create";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Create.png";
                btn.Bounds = new Rectangle(4 * 50, 0 * 50 + 75, 50, 50);
                btn.IsVisible = false;
                btn.SwitchOnAction = () =>
                {
                    //既存シーンボックスの選択解除
                    foreach (SceneBox scene2 in this.Boxes.IndexSceneboxList)
                    {
                        scene2.IsSelected = false;
                    }

                    //シーンボックス作成
                    SceneBox sceneOld = new SceneBox(this.NextId());
                    SceneBox scene = sceneOld.Clone(sceneOld.Id);
                    scene.Title = "シーン" + this.sceneCreateCounter;
                    scene.SourceBounds = new Rectangle(
                        // 画面の 200,200 あたりに作成。
                        200 + this.coordMat.SourceBounds.X % UcMain.CELL_SIZE,
                        200 + this.coordMat.SourceBounds.Y % UcMain.CELL_SIZE,
                        6 * UcMain.CELL_SIZE,
                        3 * UcMain.CELL_SIZE
                        );
                    scene.IsSelected = true;
                    UcMain.DiffContent(scene.Id, "シーンボックス#作成", scene.Diff(sceneOld));
                    this.Boxes.IndexSceneboxList.Add(scene);

                    this.sceneCreateCounter++;
                };
                //━━━━━
                // コピーボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "copy";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Copy.png";
                btn.Bounds = new Rectangle(5 * 50, 0 * 50 + 75, 50, 50);
                btn.IsVisible = false;
                //━━━━━
                // 削除ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "delete";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Delete.png";
                btn.Bounds = new Rectangle(6 * 50, 0 * 50 + 75, 50, 50);
                btn.IsVisible = false;
                //━━━━━
                // 接続ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "connect";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Connect.png";
                btn.Bounds = new Rectangle(7 * 50, 0 * 50 + 75, 50, 50);
                btn.IsVisible = false;
                btn.SwitchOnAction = () =>
                {
                    // 全シーン選択解除
                    foreach (SceneBox scene in this.Boxes.IndexSceneboxList)
                    {
                        scene.IsSelected = false;
                    }
                };
                //━━━━━
                // 4節接続ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "cable4";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Cable4.png";
                btn.Bounds = new Rectangle(8 * 50, 0 * 50 + 75, 50, 50);
                btn.IsVisible = false;
                btn.SwitchOnAction = () =>
                {
                    // 全シーン選択解除
                    foreach (SceneBox scene in this.Boxes.IndexSceneboxList)
                    {
                        scene.IsSelected = false;
                    }
                };
                //━━━━━
                // 形状切替ボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "shape";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Shape.png";
                btn.Bounds = new Rectangle(9 * 50, 0 * 50 + 75, 50, 50);
                btn.IsVisible = false;
                btn.SwitchOnAction = () =>
                {
                    // TODO:
                };
            }

            //━━━━━
            // 画像ボタン
            //━━━━━
            {
                btn = new GraphicButton(this.NextId());
                btn.Name = "image";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Image.png";
                btn.Bounds = new Rectangle(2 * 75 + 100, 0, 75, 75);
                btn.SwitchOnAction = () =>
                {
                    // ボタン表示
                    this.buttons["text"].IsVisible = true;
                    this.buttons["copy"].IsVisible = true;
                    this.buttons["delete"].IsVisible = true;
                };
                btn.RadioOffAction = () =>
                {
                    // ボタン非表示
                    this.buttons["text"].IsVisible = false;
                    this.buttons["copy"].IsVisible = false;
                    this.buttons["delete"].IsVisible = false;
                };
            }

            //━━━━━
            // コメントボタン
            //━━━━━
            {
                btn = new GraphicButton(this.NextId());
                btn.Name = "comment";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Comment.png";
                btn.Bounds = new Rectangle(3 * 75 + 100, 0, 75, 75);
                btn.SwitchOnAction = () =>
                {
                    // ボタン表示
                    this.buttons["font"].IsVisible = true;
                };
                btn.RadioOffAction = () =>
                {
                    // ボタン非表示
                    this.buttons["font"].IsVisible = false;
                };

                //━━━━━
                // フォントボタン
                //━━━━━
                btn = new GraphicButton(this.NextId());
                btn.Name = "font";
                this.buttons[btn.Name] = btn;
                btn.FilePath = "img/btn_Font.png";
                btn.Bounds = new Rectangle(3 * 50, 0 * 50 + 75, 50, 50);
                btn.IsVisible = false;
                btn.SwitchOnAction = () =>
                {
                    Actions_Text.ChangeFont(this);
                };
            }


            this.Movement = new Point();
            InitializeComponent();
            this.textBox1.Font = new Font("ＭＳ ゴシック", 12.0f);


        }

        //────────────────────────────────────────
        #endregion



        #region アクション
        //────────────────────────────────────────

        /// <summary>
        /// 
        /// </summary>
        /// <param name="id"></param>
        /// <param name="category">"シーン"とか。</param>
        public static void DiffContent(int id, string category, string diffMsg)
        {
            if (diffMsg != "")
            {
                //TODO:
                //ystem.Console.WriteLine("Diff(" + id + ")" + category + " " + diffMsg);
            }
        }

        //────────────────────────────────────────
        #endregion



        #region イベントハンドラー
        //────────────────────────────────────────


        /// <summary>
        /// クリックした地点と、マウスボタンを放した地点に丸を描き、直線で結びます。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UcMain_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;



            // 枠線
            g.DrawRectangle(Pens.Red, 0, 0, this.Width - 1, this.Height - 1);

            // 座標マット
            {
                int x = this.coordMat.SourceBounds.X;
                int y = this.coordMat.SourceBounds.Y;
                int width = this.coordMat.SourceBounds.Width;
                int height = this.coordMat.SourceBounds.Height;
                for (int row = 0; row < this.coordMatRepeatY; row++)
                {
                    for (int column = 0; column < this.coordMatRepeatX; column++)
                    {
                        this.coordMat.FileName = row + "_" + column + ".png";
                        this.coordMat.SourceBounds = new Rectangle(
                            column * width + x,
                            row * height + y,
                            width,
                            height
                            );
                        this.coordMat.Paint(g,this);
                    }
                }
                this.coordMat.SourceBounds = new Rectangle(x, y, width, height);
            }


            this.Boxes.Paint(g, this);



            // ──────────
            // 列カーソル、行カーソル
            // ──────────
            if (this.isVerticalCursorEnable)
            {
                g.FillRectangle(
                    new SolidBrush(Color.FromArgb(128, 0, 255, 0)),//半透明の緑色
                    this.verticalCursorX,
                    0,
                    UcMain.CELL_SIZE,
                    this.Height
                    );
            }

            if (this.isHorizontalCursorEnable)
            {
                g.FillRectangle(
                    new SolidBrush(Color.FromArgb(128, 0, 255, 0)),//半透明の緑色
                    0,
                    this.horizontalCursorY,
                    this.Width,
                    UcMain.CELL_SIZE
                    );
            }


            // ──────────
            // ボタン背景
            // ──────────
            g.FillRectangle(new SolidBrush(Color.FromArgb(128, 0, 0, 0)), new Rectangle(
                0,
                0,
                this.Width,
                this.buttonAreaBottom
                ));

            // 各種ボタン
            this.buttons["save"].Paint(g, this);
            this.buttons["load"].Paint(g, this);
            this.buttons["screenShot"].Paint(g, this);
            this.buttons["coordMat"].Paint(g, this);
            this.buttons["scene"].Paint(g, this);
            this.buttons["image"].Paint(g, this);
            this.buttons["comment"].Paint(g, this);
            this.buttons["move"].Paint(g, this);
            this.buttons["moveMat"].Paint(g, this);
            this.buttons["extendT"].Paint(g, this);
            this.buttons["extendR"].Paint(g, this);
            this.buttons["extendB"].Paint(g, this);
            this.buttons["extendL"].Paint(g, this);
            this.buttons["reductT"].Paint(g, this);
            this.buttons["reductR"].Paint(g, this);
            this.buttons["reductB"].Paint(g, this);
            this.buttons["reductL"].Paint(g, this);
            this.buttons["detachesH"].Paint(g, this);
            this.buttons["detachesV"].Paint(g, this);
            this.buttons["shortenH"].Paint(g, this);
            this.buttons["shortenV"].Paint(g, this);
            this.buttons["text"].Paint(g, this);
            this.buttons["create"].Paint(g, this);
            this.buttons["copy"].Paint(g, this);
            this.buttons["delete"].Paint(g, this);
            this.buttons["connect"].Paint(g, this);
            this.buttons["cable4"].Paint(g, this);
            this.buttons["shape"].Paint(g, this);
            this.buttons["font"].Paint(g, this);


            // クリック地点
            if (this.isClickedLocationVisible)
            {
                g.FillEllipse(
                    new SolidBrush(Color.FromArgb(128, 0, 0, 255)),
                    this.clickedLocation.X - 5,
                    this.clickedLocation.Y - 5,
                    10,
                    10
                    );
            }

            // マウスボタンを放した地点
            if (this.isReleaseMouseButtonLocationVisible)
            {
                g.FillEllipse(
                    Brushes.Blue,
                    this.releaseMouseButtonLocation.X - 5,
                    this.releaseMouseButtonLocation.Y - 5,
                    10,
                    10
                    );

                // 線を引きます。
                g.DrawLine(
                    Pens.Blue,
                    this.clickedLocation,
                    this.releaseMouseButtonLocation
                    );

                // 構成部品の移動量をクリアーします。
                this.coordMat.Movement = new Rectangle();
                foreach (SceneBox scene in this.Boxes.IndexSceneboxList)
                {
                    scene.Movement = new Rectangle();
                }
            }


            // ──────────
            // 画像モード
            // ──────────
            if (this.buttons["image"].IsSelected)
            {
                g.DrawString("作成：画像を画面内にドラッグ＆ドロップしてください。", this.Font, new SolidBrush(Color.FromArgb(255, 128, 255, 128)), new Point(50 + 50, 75 + 50));
            }


        }

        private void UcMain_Resize(object sender, EventArgs e)
        {
            this.Refresh();
        }

        private void UcMain_MouseUp(object sender, MouseEventArgs e)
        {

            // ──────────
            // ドラッグ＆ドロップ
            // ──────────
            this.dragAndDrop.OnMouseUp(sender, e);


            //────────────────────────────────────────
            // スイッチオン
            //────────────────────────────────────────
            #region スイッチオン

            string[] btn1 = new string[]{
                // セーブ、ロード、スクリーンショット
                "save",
                "load",
                "screenShot",
                // 座標マット、シーン、画像、コメント
                "coordMat",
                "scene",
                "image",
                "comment",
                // 移動、マット移動
                "move",
                "moveMat",
                // 拡張　上、右、下、左
                "extendT",
                "extendR",
                "extendB",
                "extendL",
                // 縮小　上、右、下、左
                "reductT",
                "reductR",
                "reductB",
                "reductL",
                // 離す、詰める
                "detachesH",
                "detachesV",
                "shortenH",
                "shortenV",
                // テキスト
                "text",
                // 作成、コピー、削除
                "create",
                "copy",
                "delete",
                // 接続、4節接続
                "connect",
                "cable4",
                // 形状切替
                "shape",
                // フォント
                "font",
            };
            foreach (string btn2 in btn1)
            {
                if (this.buttons[btn2].IsHit(e.Location))
                {
                    this.buttons[btn2].OnMouseUp(sender, e);
                }
            }

            #endregion



            // マウスボタンを放した地点を設定。
            this.releaseMouseButtonLocation = e.Location;

            bool isMoveCoordMat = false;
            int moveSceneFlg = 0;// 1:全シーン移動。2:選択シーンのみ移動。
            bool moveCableFlg = false;
            bool moveImageFlg = false;

            // 座標マットモードの場合
            if (this.buttons["coordMat"].IsSelected)
            {
                isMoveCoordMat = true;

                if (this.buttons["moveMat"].IsSelected)
                {
                    // マット移動モードの場合

                    // シーン、接続線は動かしません。
                }
                else
                {
                    moveSceneFlg = 1;
                    moveCableFlg = true;
                }
            }
            // シーンモードの場合
            if (this.buttons["scene"].IsSelected)
            {
                moveSceneFlg = 2;
            }
            // 画像モードの場合
            if (this.buttons["image"].IsSelected)
            {
                moveImageFlg = true;
            }
            // コメントモードの場合
            if (this.buttons["comment"].IsSelected)
            {
            }


            #region 移動確定
            // 選択されている構成部品があれば、移動量分だけ移動します。
            if (isMoveCoordMat)
            {
                this.coordMat.SourceBounds = new Rectangle(
                    this.coordMat.SourceBounds.X + this.Movement.X,
                    this.coordMat.SourceBounds.Y + this.Movement.Y,
                    this.coordMat.SourceBounds.Width,
                    this.coordMat.SourceBounds.Height
                    );
                this.coordMat.Movement = new Rectangle();
            }

            if (moveSceneFlg == 1)
            {
                // ──────────
                // 全シーン
                // ──────────
                this.Boxes.MoveAllScenes(this, isMoveCoordMat);
                this.Boxes.MoveAllImages(this, isMoveCoordMat);
            }
            else if (moveSceneFlg == 2)
            {
                // ──────────
                // 選択アイテム
                // ──────────
                this.Boxes.MoveSelectedScene(this);
                this.Boxes.MoveSelectedCable(this);
            }

            if (moveCableFlg)
            {
                // 全ての接続線を、移動量分だけ移動します。
                this.Boxes.MoveCable(this, isMoveCoordMat);
            }

            if (moveImageFlg)
            {
                // 全ての画像を、移動量分だけ移動します。
                //this.Boxes.MoveAllImages(this, isMoveCoordMat);
                this.Boxes.MoveSelectedImage(this);
            }

            #endregion

            this.isReleaseMouseButtonLocationVisible = true;

            this.Refresh();
        }

        private void UcMain_MouseDown(object sender, MouseEventArgs e)
        {

            //────────────────────────────────────────
            // テキストボックス→シーン名
            //────────────────────────────────────────
            if (this.textBox1.Visible)
            {
                foreach (SceneBox scene in this.Boxes.IndexSceneboxList)
                {
                    if (scene.IsSelected)
                    {
                        // タイトル確定
                        SceneBox sceneOld = scene.Clone(scene.Id);
                        scene.Title = this.textBox1.Text;
                        UcMain.DiffContent(scene.Id, "シーンボックス#タイトル確定", scene.Diff(sceneOld));
                    }
                }

                // フォーカスを外します。
                this.ActiveControl = null;
            }

            //────────────────────────────────────────
            // 何もないところでマウスボタン押下したかどうか
            //────────────────────────────────────────
            #region 何もないところでマウスボタン押下したかどうか

            bool actorPressed = false;//ボタン、シーン、接続線のいずれかの上で放したら真
            {
                string[] btn1 = new string[]{
                    // セーブ、ロード、スクリーンショット
                    "save",
                    "load",
                    "screenShot",
                    // 座標マット、シーン、画像、コメント
                    "coordMat",
                    "scene",
                    "image",
                    "comment",
                    // 移動、マット移動
                    "move",
                    "moveMat",
                    // 拡張　上、右、下、左
                    "extendT",
                    "extendR",
                    "extendB",
                    "extendL",
                    // 縮小　上、右、下、左
                    "reductT",
                    "reductR",
                    "reductB",
                    "reductL",
                    // 離す、縮める
                    "detachesH",
                    "detachesV",
                    "shortenH",
                    "shortenV",
                    // テキスト
                    "text",
                    // 作成、コピー、削除
                    "create",
                    "copy",
                    "delete",
                    // 接続、4節接続、切断
                    "connect",
                    "cable4",
                    // 形状切替
                    "shape",
                    // フォント
                    "font",
                };
                foreach (string btn2 in btn1)
                {
                    if (this.buttons[btn2].IsHit(e.Location))
                    {
                        actorPressed = true;
                    }
                }
            }


            if (this.Boxes.IsHit(e.Location))
            {
                actorPressed = true;
            }



            // 何もないところでマウスボタン押下したとき
            if (!actorPressed)
            {
                // ──────────
                // 選択解除
                // ──────────
                this.Boxes.ClearSelection();
            }

            #endregion


            // マウスボタンを押した地点を設定。
            this.clickedLocation = e.Location;
            this.isClickedLocationVisible = true;

            // マウスボタンを放した地点を消す。
            this.isReleaseMouseButtonLocationVisible = false;


            // ラジオボタンのように。
            #region ラジオボタンのように

            // 今回、マウスボタンで押されたボタン
            string pressedBtn = "";
            {
                string[] btn1 = new string[]{
                    "coordMat",
                    "scene",
                    "image",
                    "comment",
                };
                foreach (string btn2 in btn1)
                {
                    if (this.buttons[btn2].IsHit(e.Location))
                    {
                        pressedBtn = this.buttons[btn2].Name;
                    }
                }
            }

            if ("" != pressedBtn)
            {
                // 選択されていればオン、選択されていなければオフ。
                if (this.buttons["coordMat"].Name == pressedBtn)
                {
                    this.buttons["coordMat"].IsSelected = true;
                }
                else
                {
                    this.buttons["coordMat"].IsSelected = false;
                    this.buttons["coordMat"].RadioOffAction();
                }

                if (this.buttons["scene"].Name == pressedBtn)
                {
                    this.buttons["scene"].IsSelected = true;
                }
                else
                {
                    this.buttons["scene"].IsSelected = false;
                    this.buttons["scene"].RadioOffAction();
                }

                if (this.buttons["image"].Name == pressedBtn)
                {
                    this.buttons["image"].IsSelected = true;
                }
                else
                {
                    this.buttons["image"].IsSelected = false;
                    this.buttons["image"].RadioOffAction();
                }

                if (this.buttons["comment"].Name == pressedBtn)
                {
                    this.buttons["comment"].IsSelected = true;
                }
                else
                {
                    this.buttons["comment"].IsSelected = false;
                    this.buttons["comment"].RadioOffAction();
                }
            }

            // ラジオボタンのように。
            // 今回、マウスボタンで押されたボタン
            pressedBtn = "";
            {
                string[] btn1 = new string[]{
                    "move",
                    "moveMat",
                    "extendT",
                    "extendR",
                    "extendB",
                    "extendL",
                    "reductT",
                    "reductR",
                    "reductB",
                    "reductL",
                    "detachesH",
                    "detachesV",
                    "shortenH",
                    "shortenV",
                    "text",
                    "create",
                    "copy",
                    "delete",
                    "connect",
                    "cable4",
                    "shape",
                    "font",
                };
                foreach (string btn2 in btn1)
                {
                    if (this.buttons[btn2].IsHit(e.Location))
                    {
                        pressedBtn = this.buttons[btn2].Name;
                    }
                }
            }

            // 選択されていればオン、選択されていなければオフ。
            if ("" != pressedBtn)
            {
                string[] btn1 = new string[]{
                    // 移動、マット移動
                    "move",
                    "moveMat",
                    // 拡張　上、右、下、左
                    "extendT",
                    "extendR",
                    "extendB",
                    "extendL",
                    // 縮小　上、右、下、左
                    "reductT",
                    "reductR",
                    "reductB",
                    "reductL",
                    // 離す、縮める
                    "detachesH",
                    "detachesV",
                    "shortenH",
                    "shortenV",
                    // テキスト
                    "text",
                    // 作成、コピー、削除
                    "create",
                    "copy",
                    "delete",
                    // 接続、4節接続、切断
                    "connect",
                    "cable4",
                    // 形状切替
                    "shape",
                    // フォント
                    "font",
                };
                foreach (string btn2 in btn1)
                {
                    if (this.buttons[btn2].Name == pressedBtn)
                    {
                        this.buttons[btn2].IsSelected = true;
                    }
                    else
                    {
                        this.buttons[btn2].IsSelected = false;
                        this.buttons[btn2].RadioOffAction();
                    }
                }
            }

            #endregion


            // ボタンエリアと被っていない領域でマウスダウンをした場合。
            if (this.buttonAreaBottom < e.Location.Y)
            {
                if (this.buttons["scene"].IsSelected)
                {
                    if (this.buttons["connect"].IsSelected)
                    {
                        //────────────────────────────────────────
                        // シーンモード　2節接続作成モード
                        //────────────────────────────────────────
                        #region 2節接続作成モード

                        Cable cableOld = null;
                        Cable cable = null;
                        bool isNew = false;
                        if (0 < this.Boxes.IndexCableList.Count)
                        {
                            // 既存ケーブルの最終要素
                            cable = this.Boxes.IndexCableList[this.Boxes.IndexCableList.Count - 1];

                            // [0]起点　かつ　[1]終点　の両方が表示されている場合、新しく追加。
                            if (cable.IsVisible[0] && cable.IsVisible[1])
                            {
                                isNew = true;
                            }
                            else
                            {
                                cableOld = cable.Clone(cable.Id);
                            }
                        }
                        else
                        {
                            isNew = true;
                        }

                        if (isNew)
                        {
                            cableOld = new Cable(0);
                            cable = new Cable(this.NextId());
                            this.Boxes.IndexCableList.Add(cable);
                        }

                        // ──────────
                        // [0]起点　[1]終点
                        // ──────────
                        int i;
                        if (!cable.IsVisible[0])
                        {
                            i = 0;
                        }
                        else
                        {
                            i = 1;
                        }

                        cable.IsVisible[i] = true;
                        cable.SourceBounds[i] = new Rectangle(
                            (e.Location.X) - (e.Location.X - this.coordMat.Bounds.X) % UcMain.CELL_SIZE,
                            (e.Location.Y) - (e.Location.Y - this.coordMat.Bounds.Y) % UcMain.CELL_SIZE,
                            UcMain.CELL_SIZE,
                            UcMain.CELL_SIZE
                            );
                        UcMain.DiffContent(cable.Id, "接続線2節#作成", cable.Diff(cableOld));

                        #endregion
                    }
                    else if (this.buttons["cable4"].IsSelected)
                    {
                        //────────────────────────────────────────
                        // シーンモード　4節接続作成モード
                        //────────────────────────────────────────
                        #region 4節接続作成モード

                        Cable cableOld = new Cable(this.NextId());
                        Cable cable = cableOld.Clone(cableOld.Id);

                        // 始点
                        cable.IsVisible[0] = true;
                        cable.SourceBounds[0] = new Rectangle(
                            (e.Location.X) - (e.Location.X - this.coordMat.Bounds.X) % UcMain.CELL_SIZE,
                            (e.Location.Y) - (e.Location.Y - this.coordMat.Bounds.Y) % UcMain.CELL_SIZE,
                            UcMain.CELL_SIZE,
                            UcMain.CELL_SIZE
                            );

                        // 中間点0
                        cable.IsVisibleNode[0] = true;
                        cable.SourceBoundsNode[0] = new Point(
                            (e.Location.X) - (e.Location.X - this.coordMat.Bounds.X) % UcMain.CELL_SIZE + 2 * UcMain.CELL_SIZE,
                            (e.Location.Y) - (e.Location.Y - this.coordMat.Bounds.Y) % UcMain.CELL_SIZE + 2 * UcMain.CELL_SIZE
                            );

                        // 中間点1
                        cable.IsVisibleNode[1] = true;
                        cable.SourceBoundsNode[1] = new Point(
                            (e.Location.X) - (e.Location.X - this.coordMat.Bounds.X) % UcMain.CELL_SIZE - 4 * UcMain.CELL_SIZE,
                            (e.Location.Y) - (e.Location.Y - this.coordMat.Bounds.Y) % UcMain.CELL_SIZE - 2 * UcMain.CELL_SIZE
                            );

                        // 終点
                        cable.IsVisible[1] = true;
                        cable.SourceBounds[1] = new Rectangle(
                            (e.Location.X) - (e.Location.X - this.coordMat.Bounds.X) % UcMain.CELL_SIZE - 2 * UcMain.CELL_SIZE,
                            (e.Location.Y) - (e.Location.Y - this.coordMat.Bounds.Y) % UcMain.CELL_SIZE - 2 * UcMain.CELL_SIZE,
                            UcMain.CELL_SIZE,
                            UcMain.CELL_SIZE
                            );

                        UcMain.DiffContent(cable.Id, "接続線4節#作成", cable.Diff(cableOld));
                        this.Boxes.IndexCableList.Add(cable);
                        #endregion
                    }
                    else
                    {
                        //────────────────────────────────────────
                        // シーンモード　その他
                        //────────────────────────────────────────

                        // ──────────
                        // 排他選択
                        // ──────────
                        // (1)接続線 (2)シーン
                        #region 接続線_選択
                        // 始端終端
                        SceneBox scene2 = null;
                        Cable cable2 = null;
                        int i = 0;
                        foreach (Cable cable in this.Boxes.IndexCableList)
                        {
                            if (cable.IsHit(1, e.Location))
                            {
                                cable2 = cable;
                                i = 1;
                                // 最初の１件のみ
                                break;
                            }
                            else if (cable.IsHit(0, e.Location))
                            {
                                cable2 = cable;
                                i = 0;
                                // 最初の１件のみ
                                break;
                            }
                        }

                        if (null != cable2)
                        {
                            cable2.IsSelected[i] = true;
                            goto END_CABLE_SELECT;
                        }

                        // 中間点2つ
                        cable2 = null;
                        i = 0;
                        foreach (Cable cable in this.Boxes.IndexCableList)
                        {
                            if (cable.IsHitNode(1, e.Location))
                            {
                                cable2 = cable;
                                i = 1;
                                // 最初の１件のみ
                                break;
                            }
                            else if (cable.IsHitNode(0, e.Location))
                            {
                                cable2 = cable;
                                i = 0;
                                // 最初の１件のみ
                                break;
                            }
                        }

                        if (null != cable2)
                        {
                            cable2.IsSelectedNode[i] = true;
                            goto END_CABLE_SELECT;
                        }

                        // ──────────
                        // 選択シーンボックス
                        // ──────────
                        foreach (SceneBox scene in this.Boxes.IndexSceneboxList)
                        {
                            if (scene.IsHit(e.Location))
                            {
                                scene2 = scene;
                                // 最初の１件のみ
                                break;
                            }
                        }

                    END_CABLE_SELECT:
                        ;
                        #endregion


                        if (this.buttons["copy"].IsSelected)
                        {
                            if (null != scene2)
                            {
                                //────────────────────────────────────────
                                // コピー
                                //────────────────────────────────────────
                                #region コピー

                                SceneBox scene3 = scene2.Clone(this.NextId());
                                SceneBox sceneOld = new SceneBox(0);
                                scene3.SourceBounds = new Rectangle(
                                    scene3.SourceBounds.X + UcMain.CELL_SIZE,
                                    scene3.SourceBounds.Y + UcMain.CELL_SIZE,
                                    scene3.SourceBounds.Width,
                                    scene3.SourceBounds.Height
                                    );
                                UcMain.DiffContent(scene3.Id, "シーンボックス#コピー", scene3.Diff(sceneOld));
                                this.Boxes.IndexSceneboxList.Add(scene3);

                                #endregion
                            }

                        }
                        else if (this.buttons["shape"].IsSelected)
                        {
                            if (null != scene2)
                            {
                                //────────────────────────────────────────
                                // 形状切替
                                //────────────────────────────────────────
                                #region 形状切替

                                SceneBox sceneOld = scene2.Clone(scene2.Id);

                                switch (scene2.Shape)
                                {
                                    case 0:
                                        scene2.Shape = 1;
                                        break;
                                    case 1:
                                        scene2.Shape = 2;
                                        break;
                                    default:
                                        scene2.Shape = 0;
                                        break;
                                }

                                UcMain.DiffContent(scene2.Id, "シーンボックス#形状切替", scene2.Diff(sceneOld));

                                #endregion
                            }

                        }
                        else if (this.buttons["text"].IsSelected)
                        {
                            if (null != scene2)
                            {
                                //────────────────────────────────────────
                                // テキストモード
                                //────────────────────────────────────────
                                #region テキストモード

                                //他のシーンを選択解除
                                foreach (SceneBox scene in this.Boxes.IndexSceneboxList)
                                {
                                    scene.IsSelected = false;
                                }

                                scene2.IsSelected = true;
                                this.textBox1.Visible = true;
                                this.textBox1.Bounds = new Rectangle(
                                    scene2.Bounds.X,
                                    scene2.Bounds.Y,
                                    scene2.Bounds.Width,
                                    scene2.Bounds.Height
                                    );
                                this.textBox1.Text = scene2.Title;
                                this.textBox1.Font = this.Font;

                                this.textBox1.Focus();
                                this.textBox1.SelectAll();

                                #endregion
                            }

                        }
                        else if (this.buttons["delete"].IsSelected)
                        {
                            //────────────────────────────────────────
                            // 削除モード
                            //────────────────────────────────────────
                            #region 削除モード

                            if (null != cable2)
                            {
                                this.Boxes.IndexCableList.Remove(cable2);
                                UcMain.DiffContent(cable2.Id, "接続線#削除", cable2.Now());
                            }

                            if (null != scene2)
                            {
                                this.Boxes.IndexSceneboxList.Remove(scene2);
                                UcMain.DiffContent(scene2.Id, "シーンボックス#削除", scene2.Now());
                            }

                            #endregion
                        }
                        else
                        {
                            if (null != scene2)
                            {
                                scene2.IsSelected = true;
                            }
                        }

                    }
                }
                else if (this.buttons["coordMat"].IsSelected)
                {
                    if (this.buttons["detachesH"].IsSelected)
                    {
                        //────────────────────────────────────────
                        // 座標マットモード　離す横
                        //────────────────────────────────────────

                        // マウスクリック位置のグリッド列以右のものを、グリッド１つ分右にずらします。
                        Actions_Coordmat.DetachesH(e.Location, this);
                    }
                    else if (this.buttons["detachesV"].IsSelected)
                    {
                        //────────────────────────────────────────
                        // 座標マットモード　離す縦
                        //────────────────────────────────────────

                        // マウスクリック位置のグリッド行以下のものを、グリッド１つ分下げます。
                        Actions_Coordmat.DetachesV(e.Location, this);
                    }
                    else if (this.buttons["shortenH"].IsSelected)
                    {
                        //────────────────────────────────────────
                        // 座標マットモード　縮める横
                        //────────────────────────────────────────

                        // マウスクリック位置のグリッド列以右のものを、グリッド１つ分左にずらします。
                        Actions_Coordmat.ShortenH(e.Location,this);
                    }
                    else if (this.buttons["shortenV"].IsSelected)
                    {
                        //────────────────────────────────────────
                        // 座標マットモード　縮める縦
                        //────────────────────────────────────────

                        // マウスクリック位置のグリッド行以下のものを、グリッド１つ分下げます。
                        Actions_Coordmat.ShortenV(e.Location, this);
                    }
                    else
                    {
                        this.isVerticalCursorEnable = false;
                        this.isHorizontalCursorEnable = false;
                    }
                }
                else if (this.buttons["image"].IsSelected)
                {
                    //────────────────────────────────────────
                    // 画像モード　その他
                    //────────────────────────────────────────

                    // ──────────
                    // 排他選択
                    // ──────────
                    #region 選択
                    // 始端終端
                    ImageBox ib2 = null;
                    int i = 0;

                    foreach (ImageBox ib in this.Boxes.IndexImageboxList)
                    {
                        if (ib.IsHit(e.Location))
                        {
                            ib2 = ib;
                            // 最初の１件のみ
                            break;
                        }
                    }

                    if (null != ib2)
                    {
                        ib2.IsSelected = true;
                    }

                    #endregion

                }
                else if (this.buttons["comment"].IsSelected)
                {
                }
            }

            // ──────────
            // ドラッグ＆ドロップ
            // ──────────
            this.dragAndDrop.OnMouseDown(sender, e);

            this.Refresh();
        }

        private void UcMain_Load(object sender, EventArgs e)
        {
            this.buttons["save"].Load();
            this.buttons["load"].Load();
            this.buttons["screenShot"].Load();

            this.buttons["coordMat"].Load();
            this.buttons["scene"].Load();
            this.buttons["image"].Load();
            this.buttons["comment"].Load();

            this.buttons["move"].Load();
            this.buttons["moveMat"].Load();
            this.buttons["extendT"].Load();
            this.buttons["extendR"].Load();
            this.buttons["extendB"].Load();
            this.buttons["extendL"].Load();
            this.buttons["reductT"].Load();
            this.buttons["reductR"].Load();
            this.buttons["reductB"].Load();
            this.buttons["reductL"].Load();
            this.buttons["detachesH"].Load();
            this.buttons["detachesV"].Load();
            this.buttons["shortenH"].Load();
            this.buttons["shortenV"].Load();
            this.buttons["text"].Load();
            this.buttons["create"].Load();
            this.buttons["copy"].Load();
            this.buttons["delete"].Load();
            this.buttons["connect"].Load();
            this.buttons["cable4"].Load();
            this.buttons["shape"].Load();
            this.buttons["font"].Load();
        }

        private void UcMain_MouseMove(object sender, MouseEventArgs e)
        {
            // マウスオーバー
            #region マウスオーバー

            bool forcedOff = false;
            if (this.buttons["coordMat"].IsSelected)
            {
                //────────────────────────────────────────
                // 座標マットモード
                //────────────────────────────────────────
            }
            else if (this.buttons["scene"].IsSelected)
            {
                //────────────────────────────────────────
                // シーンモード
                //────────────────────────────────────────
                // ・接続線
                // ・シーン
                //にマウスカーソルが合わさったとき、枠線を緑色にします。
                this.Boxes.CheckMouseOver(e.Location, ref forcedOff);

            }
            else if (this.buttons["image"].IsSelected)
            {
                //────────────────────────────────────────
                // 画像モード
                //────────────────────────────────────────
                this.Boxes.CheckMouseOver(e.Location, ref forcedOff);
            }
            else if (this.buttons["comment"].IsSelected)
            {
                //────────────────────────────────────────
                // コメントモード
                //────────────────────────────────────────
            }

            //────────────────────────────────────────
            // 優先順3　ボタン
            //────────────────────────────────────────

            // セーブ、ロード、スクリーンショット
            this.buttons["save"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["load"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["screenShot"].CheckMouseOver(e.Location, ref forcedOff);

            // 座標マット、シーン、画像、コメント
            this.buttons["coordMat"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["scene"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["image"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["comment"].CheckMouseOver(e.Location, ref forcedOff);

            // 移動
            this.buttons["move"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["moveMat"].CheckMouseOver(e.Location, ref forcedOff);

            // 拡張　上、右、下、左
            this.buttons["extendT"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["extendR"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["extendB"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["extendL"].CheckMouseOver(e.Location, ref forcedOff);

            // 縮小　上、右、下、左
            this.buttons["reductT"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["reductR"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["reductB"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["reductL"].CheckMouseOver(e.Location, ref forcedOff);

            // 離す、縮める
            this.buttons["detachesH"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["detachesV"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["shortenH"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["shortenV"].CheckMouseOver(e.Location, ref forcedOff);

            // テキスト
            this.buttons["text"].CheckMouseOver(e.Location, ref forcedOff);

            // 作成、コピー、削除
            this.buttons["create"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["copy"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["delete"].CheckMouseOver(e.Location, ref forcedOff);

            // 接続、4節接続
            this.buttons["connect"].CheckMouseOver(e.Location, ref forcedOff);
            this.buttons["cable4"].CheckMouseOver(e.Location, ref forcedOff);

            // 形状切替
            this.buttons["shape"].CheckMouseOver(e.Location, ref forcedOff);

            // フォント
            this.buttons["font"].CheckMouseOver(e.Location, ref forcedOff);

            #endregion


            if (this.buttons["text"].IsSelected)
            {
                //────────────────────────────────────────
                // テキストモード
                //────────────────────────────────────────

                // 移動モード禁止。
            }
            else
            {
                if (this.isClickedLocationVisible && !this.isReleaseMouseButtonLocationVisible)
                {
                    // マウス・ドラッグの移動量を測ります。
                    if (this.buttons["moveMat"].IsSelected)
                    {
                        // マット移動モードの場合、グリッド単位で移動します。
                        this.Movement = new Point(
                            e.Location.X - this.clickedLocation.X - (e.Location.X - this.clickedLocation.X) % UcMain.CELL_SIZE,
                            e.Location.Y - this.clickedLocation.Y - (e.Location.Y - this.clickedLocation.Y) % UcMain.CELL_SIZE
                            );
                    }
                    else
                    {
                        this.Movement = new Point(
                            e.Location.X - this.clickedLocation.X,
                            e.Location.Y - this.clickedLocation.Y
                            );
                    }
                    //ystem.Console.WriteLine("ドラッグ中 e(" + e.Location.X + "," + e.Location.Y + ") click(" + this.clickedLocation.X + "," + this.clickedLocation.Y + ") move(" + this.Movement.X + "," + this.Movement.Y + ")");

                    
                    //マットを動かすのか否か
                    bool isMoveCoordmat = false;

                    int moveSceneFlg = 0;// 1:全シーン移動。2:選択シーンのみ移動。
                    bool moveCableFlg = false;// ケーブルの移動か
                    bool moveImageFlg = false;// 画像の移動か
                    // 座標マットモードの場合
                    if (this.buttons["coordMat"].IsSelected)
                    {
                        //マットを動かします。
                        isMoveCoordmat = true;

                        if (this.buttons["moveMat"].IsSelected)
                        {
                            // マット移動モードの場合

                            // シーン、接続線は動かしません。
                        }
                        else
                        {
                            moveSceneFlg = 1;//全シーン移動
                            moveCableFlg = true;
                        }
                    }
                    // シーンモードの場合
                    if (this.buttons["scene"].IsSelected)
                    {
                        moveSceneFlg = 2;//選択シーンのみ移動
                    }
                    else if (this.buttons["image"].IsSelected)
                    {
                        moveImageFlg = true;
                    }
                    else if (this.buttons["comment"].IsSelected)
                    {
                    }


                    // 選択している構成部品に、移動量をセットします。
                    if (isMoveCoordmat)
                    {
                        //マットを動かします。
                        this.coordMat.Movement = new Rectangle(this.Movement.X, this.Movement.Y, 0, 0);
                    }

                    if (moveSceneFlg == 1)//全シーン移動
                    {
                        // ──────────
                        // 全シーン　移動量セット
                        // ──────────
                        Boxes.SetMovementToAllSceneboxes(this, isMoveCoordmat);

                        Boxes.SetMovementToAllImageboxes(this, isMoveCoordmat);
                    }
                    else if (moveSceneFlg == 2)//選択シーンのみ移動
                    {
                        // ──────────
                        // 選択アイテムの移動量確定
                        // ──────────
                        Boxes.SetMovementToSelectedScenebox(this);

                        Boxes.SetMovementToSelectedCable(this);
                    }

                    if (moveCableFlg)
                    {
                        // ──────────
                        // 全接続線の移動量確定
                        // ──────────
                        Boxes.SetMovementToAllCables(this, isMoveCoordmat);
                    }

                    if (moveImageFlg)//選択画像の移動
                    {
                        Boxes.SetMovementToSelectedImagebox(this);
                    }
                }
                else
                {
                    this.Movement = new Point();
                }
            }


            if (this.buttons["coordMat"].IsSelected)
            {
                if (this.buttons["detachesH"].IsSelected)
                {
                    //────────────────────────────────────────
                    // 座標マットモード　離す横
                    //────────────────────────────────────────
                    #region 座標マットモード　離す横

                    // ──────────
                    // マウスオーバー列
                    // ──────────
                    this.verticalCursorX = (e.Location.X - this.coordMat.Bounds.X) / UcMain.CELL_SIZE * UcMain.CELL_SIZE + this.coordMat.Bounds.X;
                    this.isVerticalCursorEnable = true;
                    this.isHorizontalCursorEnable = false;

                    #endregion
                }
                else if (this.buttons["detachesV"].IsSelected)
                {
                    //────────────────────────────────────────
                    // 座標マットモード　離す縦
                    //────────────────────────────────────────
                    #region 座標マットモード　離す縦

                    // ──────────
                    // マウスオーバー行
                    // ──────────
                    this.horizontalCursorY = (e.Location.Y - this.coordMat.Bounds.Y) / UcMain.CELL_SIZE * UcMain.CELL_SIZE + this.coordMat.Bounds.Y;
                    this.isHorizontalCursorEnable = true;
                    this.isVerticalCursorEnable = false;

                    #endregion
                }
                else if (this.buttons["shortenH"].IsSelected)
                {
                    //────────────────────────────────────────
                    // 座標マットモード　縮める横
                    //────────────────────────────────────────
                    #region 座標マットモード　縮める横

                    // ──────────
                    // マウスオーバー列
                    // ──────────
                    this.verticalCursorX = (e.Location.X - this.coordMat.Bounds.X) / UcMain.CELL_SIZE * UcMain.CELL_SIZE + this.coordMat.Bounds.X;
                    this.isVerticalCursorEnable = true;
                    this.isHorizontalCursorEnable = false;

                    #endregion
                }
                else if (this.buttons["shortenV"].IsSelected)
                {
                    //────────────────────────────────────────
                    // 座標マットモード　縮める縦
                    //────────────────────────────────────────
                    #region 座標マットモード　縮める縦

                    // ──────────
                    // マウスオーバー行
                    // ──────────
                    this.horizontalCursorY = (e.Location.Y - this.coordMat.Bounds.Y) / UcMain.CELL_SIZE * UcMain.CELL_SIZE + this.coordMat.Bounds.Y;
                    this.isHorizontalCursorEnable = true;
                    this.isVerticalCursorEnable = false;

                    #endregion
                }
                else
                {
                    this.isVerticalCursorEnable = false;
                    this.isHorizontalCursorEnable = false;
                }
            }

            this.Refresh();
        }

        private void textBox1_Leave(object sender, EventArgs e)
        {
            this.textBox1.Visible = false;
        }

        private void UcMain_QueryContinueDrag(object sender, QueryContinueDragEventArgs e)
        {
            this.dragAndDrop.OnQueryContinueDrag(sender, e);
        }

        private void UcMain_DragDrop(object sender, DragEventArgs e)
        {
            //━━━━━
            //画像をドロップ
            //━━━━━
            System.Console.WriteLine("★★★★★★★★UiMain_DragDrop");

            this.dragAndDrop.OnDragDrop(sender, e);
        }

        private void UcMain_DragEnter(object sender, DragEventArgs e)
        {
            System.Console.WriteLine("★★★★★★★★UiMain_DragEnter");

            //this.Cursor = new Cursor("img/DragMoveImage.ico");
            //this.Refresh();
            //Cursor.Current = new Cursor("img/DragMoveImage.ico");

            this.dragAndDrop.OnDragEnter(sender, e);
        }

        private void UcMain_MouseEnter(object sender, EventArgs e)
        {
            //System.Console.WriteLine("★★★★★★★★UiMain_MouseEnter");
        }

        private void UcMain_DragOver(object sender, DragEventArgs e)
        {
            //this.Cursor = new Cursor("img/DragMoveImage.ico");
        }

        //────────────────────────────────────────
        #endregion


        #region プロパティー
        //────────────────────────────────────────

        /// <summary>
        /// 最後に作成したコンテンツに割り振った数字です。重複させません。
        /// 初期値は 0 ですが、NextId()関数を使って割り振るので、実際に使われるのは 1～ です。
        /// </summary>
        private int lastId;
        private int NextId()
        {
            this.lastId++;
            return this.lastId;
        }

        //────────────────────────────────────────

        /// <summary>
        /// 座標マット。
        /// </summary>
        private CoordMat coordMat;
        public CoordMat CoordMat
        {
            get
            {
                return coordMat;
            }
        }

        private int coordMatRepeatX;
        public int CoordMatRepeatX
        {
            get
            {
                return this.coordMatRepeatX;
            }
        }

        private int coordMatRepeatY;
        public int CoordMatRepeatY
        {
            get
            {
                return this.coordMatRepeatY;
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// クリックした始点。
        /// </summary>
        private Point clickedLocation;
        private bool isClickedLocationVisible;

        /// <summary>
        /// マウスボタンを離した地点。
        /// </summary>
        private Point releaseMouseButtonLocation;
        private bool isReleaseMouseButtonLocationVisible;

        /// <summary>
        /// 画像ボタン。
        /// </summary>
        private Dictionary<string, GraphicButton> buttons;

        /// <summary>
        /// 配置物一覧。
        /// </summary>
        private Boxes boxes;
        public Boxes Boxes
        {
            get
            {
                return this.boxes;
            }
            set
            {
                this.boxes = value;
            }
        }

        //────────────────────────────────────────

        /// <summary>
        /// シーン作成回数。0スタート。
        /// </summary>
        private int sceneCreateCounter;

        /// <summary>
        /// マウス・ドラッグの移動量を測ります。
        /// </summary>
        private Point movement;
        public Point Movement
        {
            get
            {
                return movement;
            }
            set
            {
                movement = value;
            }
        }

        /// <summary>
        /// ボタン領域の下端。
        /// </summary>
        private int buttonAreaBottom;

        /// <summary>
        /// 水平線グリッドカーソル。
        /// </summary>
        private int horizontalCursorY;
        private bool isHorizontalCursorEnable;
        /// <summary>
        /// 垂直線グリッドカーソル。
        /// </summary>
        private int verticalCursorX;
        private bool isVerticalCursorEnable;

        /// <summary>
        /// IDを表示するなら真。主にデバッグ用。
        /// </summary>
        private static bool visibleId;
        public static bool VisibleId
        {
            get
            {
                return visibleId;
            }
            set
            {
                visibleId = value;
            }
        }

        /// <summary>
        /// ドラッグ＆ドロップのプログラム。
        /// </summary>
        private DragAndDrop dragAndDrop;

        /// <summary>
        /// ファイル名を作成するときに、重複しないように数字をカウントしておきます。
        /// </summary>
        private int fileNameCounter;

        //────────────────────────────────────────
        #endregion


    }
}
